# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The freesurfer module provides basic functions for interfacing with
   freesurfer tools.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ...utils.filemanip import fname_presuffix, split_filename
from ..base import (TraitedSpec, File, traits, InputMultiPath, OutputMultiPath,
                    Directory, isdefined)
from .base import FSCommand, FSTraitedSpec
from .utils import copy2subjdir

__docformat__ = 'restructuredtext'


class MRISPreprocInputSpec(FSTraitedSpec):
    out_file = File(argstr='--out %s', genfile=True, desc='output filename')
    target = traits.Str(
        argstr='--target %s', mandatory=True, desc='target subject name')
    hemi = traits.Enum(
        'lh',
        'rh',
        argstr='--hemi %s',
        mandatory=True,
        desc='hemisphere for source and target')
    surf_measure = traits.Str(
        argstr='--meas %s',
        xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        desc='Use subject/surf/hemi.surf_measure as input')
    surf_area = traits.Str(
        argstr='--area %s',
        xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        desc=
        'Extract vertex area from subject/surf/hemi.surfname to use as input.')
    subjects = traits.List(
        argstr='--s %s...',
        xor=('subjects', 'fsgd_file', 'subject_file'),
        desc='subjects from who measures are calculated')
    fsgd_file = File(
        exists=True,
        argstr='--fsgd %s',
        xor=('subjects', 'fsgd_file', 'subject_file'),
        desc='specify subjects using fsgd file')
    subject_file = File(
        exists=True,
        argstr='--f %s',
        xor=('subjects', 'fsgd_file', 'subject_file'),
        desc='file specifying subjects separated by white space')
    surf_measure_file = InputMultiPath(
        File(exists=True),
        argstr='--is %s...',
        xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        desc='file alternative to surfmeas, still requires list of subjects')
    source_format = traits.Str(argstr='--srcfmt %s', desc='source format')
    surf_dir = traits.Str(
        argstr='--surfdir %s', desc='alternative directory (instead of surf)')
    vol_measure_file = InputMultiPath(
        traits.Tuple(File(exists=True), File(exists=True)),
        argstr='--iv %s %s...',
        desc='list of volume measure and reg file tuples')
    proj_frac = traits.Float(
        argstr='--projfrac %s', desc='projection fraction for vol2surf')
    fwhm = traits.Float(
        argstr='--fwhm %f',
        xor=['num_iters'],
        desc='smooth by fwhm mm on the target surface')
    num_iters = traits.Int(
        argstr='--niters %d',
        xor=['fwhm'],
        desc='niters : smooth by niters on the target surface')
    fwhm_source = traits.Float(
        argstr='--fwhm-src %f',
        xor=['num_iters_source'],
        desc='smooth by fwhm mm on the source surface')
    num_iters_source = traits.Int(
        argstr='--niterssrc %d',
        xor=['fwhm_source'],
        desc='niters : smooth by niters on the source surface')
    smooth_cortex_only = traits.Bool(
        argstr='--smooth-cortex-only',
        desc='only smooth cortex (ie, exclude medial wall)')


class MRISPreprocOutputSpec(TraitedSpec):
    out_file = File(desc='preprocessed output file')


class MRISPreproc(FSCommand):
    """Use FreeSurfer mris_preproc to prepare a group of contrasts for
    a second level analysis

    Examples
    --------

    >>> preproc = MRISPreproc()
    >>> preproc.inputs.target = 'fsaverage'
    >>> preproc.inputs.hemi = 'lh'
    >>> preproc.inputs.vol_measure_file = [('cont1.nii', 'register.dat'), \
                                           ('cont1a.nii', 'register.dat')]
    >>> preproc.inputs.out_file = 'concatenated_file.mgz'
    >>> preproc.cmdline
    'mris_preproc --hemi lh --out concatenated_file.mgz --target fsaverage --iv cont1.nii register.dat --iv cont1a.nii register.dat'

    """

    _cmd = 'mris_preproc'
    input_spec = MRISPreprocInputSpec
    output_spec = MRISPreprocOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outfile = self.inputs.out_file
        outputs['out_file'] = outfile
        if not isdefined(outfile):
            outputs['out_file'] = os.path.join(
                os.getcwd(), 'concat_%s_%s.mgz' % (self.inputs.hemi,
                                                   self.inputs.target))
        return outputs

    def _gen_filename(self, name):
        if name == 'out_file':
            return self._list_outputs()[name]
        return None


class MRISPreprocReconAllInputSpec(MRISPreprocInputSpec):
    surf_measure_file = File(
        exists=True,
        argstr='--meas %s',
        xor=('surf_measure', 'surf_measure_file', 'surf_area'),
        desc='file necessary for surfmeas')
    surfreg_files = InputMultiPath(
        File(exists=True),
        argstr="--surfreg %s",
        requires=['lh_surfreg_target', 'rh_surfreg_target'],
        desc="lh and rh input surface registration files")
    lh_surfreg_target = File(
        desc="Implicit target surface registration file",
        requires=['surfreg_files'])
    rh_surfreg_target = File(
        desc="Implicit target surface registration file",
        requires=['surfreg_files'])
    subject_id = traits.String(
        'subject_id',
        argstr='--s %s',
        usedefault=True,
        xor=('subjects', 'fsgd_file', 'subject_file', 'subject_id'),
        desc='subject from whom measures are calculated')
    copy_inputs = traits.Bool(
        desc="If running as a node, set this to True " +
        "this will copy some implicit inputs to the " + "node directory.")


class MRISPreprocReconAll(MRISPreproc):
    """Extends MRISPreproc to allow it to be used in a recon-all workflow

    Examples
    ========
    >>> preproc = MRISPreprocReconAll()
    >>> preproc.inputs.target = 'fsaverage'
    >>> preproc.inputs.hemi = 'lh'
    >>> preproc.inputs.vol_measure_file = [('cont1.nii', 'register.dat'), \
                                           ('cont1a.nii', 'register.dat')]
    >>> preproc.inputs.out_file = 'concatenated_file.mgz'
    >>> preproc.cmdline
    'mris_preproc --hemi lh --out concatenated_file.mgz --s subject_id --target fsaverage --iv cont1.nii register.dat --iv cont1a.nii register.dat'
    """

    input_spec = MRISPreprocReconAllInputSpec

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if 'subjects_dir' in inputs:
                inputs['subjects_dir'] = self.inputs.subjects_dir
            if isdefined(self.inputs.surf_dir):
                folder = self.inputs.surf_dir
            else:
                folder = 'surf'
            if isdefined(self.inputs.surfreg_files):
                for surfreg in self.inputs.surfreg_files:
                    basename = os.path.basename(surfreg)
                    copy2subjdir(self, surfreg, folder, basename)
                    if basename.startswith('lh.'):
                        copy2subjdir(
                            self,
                            self.inputs.lh_surfreg_target,
                            folder,
                            basename,
                            subject_id=self.inputs.target)
                    else:
                        copy2subjdir(
                            self,
                            self.inputs.rh_surfreg_target,
                            folder,
                            basename,
                            subject_id=self.inputs.target)

            if isdefined(self.inputs.surf_measure_file):
                copy2subjdir(self, self.inputs.surf_measure_file, folder)

        return super(MRISPreprocReconAll, self).run(**inputs)

    def _format_arg(self, name, spec, value):
        # mris_preproc looks for these files in the surf dir
        if name == 'surfreg_files':
            basename = os.path.basename(value[0])
            return spec.argstr % basename.lstrip('rh.').lstrip('lh.')
        if name == "surf_measure_file":
            basename = os.path.basename(value)
            return spec.argstr % basename.lstrip('rh.').lstrip('lh.')
        return super(MRISPreprocReconAll, self)._format_arg(name, spec, value)


class GLMFitInputSpec(FSTraitedSpec):
    glm_dir = traits.Str(
        argstr='--glmdir %s', desc='save outputs to dir', genfile=True)
    in_file = File(
        desc='input 4D file', argstr='--y %s', mandatory=True, copyfile=False)
    _design_xor = ('fsgd', 'design', 'one_sample')
    fsgd = traits.Tuple(
        File(exists=True),
        traits.Enum('doss', 'dods'),
        argstr='--fsgd %s %s',
        xor=_design_xor,
        desc='freesurfer descriptor file')
    design = File(
        exists=True,
        argstr='--X %s',
        xor=_design_xor,
        desc='design matrix file')
    contrast = InputMultiPath(
        File(exists=True), argstr='--C %s...', desc='contrast file')

    one_sample = traits.Bool(
        argstr='--osgm',
        xor=('one_sample', 'fsgd', 'design', 'contrast'),
        desc='construct X and C as a one-sample group mean')
    no_contrast_ok = traits.Bool(
        argstr='--no-contrasts-ok',
        desc='do not fail if no contrasts specified')
    per_voxel_reg = InputMultiPath(
        File(exists=True), argstr='--pvr %s...', desc='per-voxel regressors')
    self_reg = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        argstr='--selfreg %d %d %d',
        desc='self-regressor from index col row slice')
    weighted_ls = File(
        exists=True,
        argstr='--wls %s',
        xor=('weight_file', 'weight_inv', 'weight_sqrt'),
        desc='weighted least squares')
    fixed_fx_var = File(
        exists=True, argstr='--yffxvar %s', desc='for fixed effects analysis')
    fixed_fx_dof = traits.Int(
        argstr='--ffxdof %d',
        xor=['fixed_fx_dof_file'],
        desc='dof for fixed effects analysis')
    fixed_fx_dof_file = File(
        argstr='--ffxdofdat %d',
        xor=['fixed_fx_dof'],
        desc='text file with dof for fixed effects analysis')
    weight_file = File(
        exists=True,
        xor=['weighted_ls'],
        desc='weight for each input at each voxel')
    weight_inv = traits.Bool(
        argstr='--w-inv', desc='invert weights', xor=['weighted_ls'])
    weight_sqrt = traits.Bool(
        argstr='--w-sqrt', desc='sqrt of weights', xor=['weighted_ls'])
    fwhm = traits.Range(
        low=0.0, argstr='--fwhm %f', desc='smooth input by fwhm')
    var_fwhm = traits.Range(
        low=0.0, argstr='--var-fwhm %f', desc='smooth variance by fwhm')
    no_mask_smooth = traits.Bool(
        argstr='--no-mask-smooth', desc='do not mask when smoothing')
    no_est_fwhm = traits.Bool(
        argstr='--no-est-fwhm', desc='turn off FWHM output estimation')
    mask_file = File(exists=True, argstr='--mask %s', desc='binary mask')
    label_file = File(
        exists=True,
        argstr='--label %s',
        xor=['cortex'],
        desc='use label as mask, surfaces only')
    cortex = traits.Bool(
        argstr='--cortex',
        xor=['label_file'],
        desc='use subjects ?h.cortex.label as label')
    invert_mask = traits.Bool(argstr='--mask-inv', desc='invert mask')
    prune = traits.Bool(
        argstr='--prune',
        desc=
        'remove voxels that do not have a non-zero value at each frame (def)')
    no_prune = traits.Bool(
        argstr='--no-prune', xor=['prunethresh'], desc='do not prune')
    prune_thresh = traits.Float(
        argstr='--prune_thr %f',
        xor=['noprune'],
        desc='prune threshold. Default is FLT_MIN')
    compute_log_y = traits.Bool(
        argstr='--logy', desc='compute natural log of y prior to analysis')
    save_estimate = traits.Bool(
        argstr='--yhat-save', desc='save signal estimate (yhat)')
    save_residual = traits.Bool(
        argstr='--eres-save', desc='save residual error (eres)')
    save_res_corr_mtx = traits.Bool(
        argstr='--eres-scm',
        desc='save residual error spatial correlation matrix (eres.scm). Big!')
    surf = traits.Bool(
        argstr="--surf %s %s %s",
        requires=["subject_id", "hemi"],
        desc="analysis is on a surface mesh")
    subject_id = traits.Str(desc="subject id for surface geometry")
    hemi = traits.Enum("lh", "rh", desc="surface hemisphere")
    surf_geo = traits.Str(
        "white",
        usedefault=True,
        desc="surface geometry name (e.g. white, pial)")
    simulation = traits.Tuple(
        traits.Enum('perm', 'mc-full', 'mc-z'),
        traits.Int(min=1),
        traits.Float,
        traits.Str,
        argstr='--sim %s %d %f %s',
        desc='nulltype nsim thresh csdbasename')
    sim_sign = traits.Enum(
        'abs', 'pos', 'neg', argstr='--sim-sign %s', desc='abs, pos, or neg')
    uniform = traits.Tuple(
        traits.Float,
        traits.Float,
        argstr='--uniform %f %f',
        desc='use uniform distribution instead of gaussian')
    pca = traits.Bool(
        argstr='--pca', desc='perform pca/svd analysis on residual')
    calc_AR1 = traits.Bool(
        argstr='--tar1', desc='compute and save temporal AR1 of residual')
    save_cond = traits.Bool(
        argstr='--save-cond',
        desc='flag to save design matrix condition at each voxel')
    vox_dump = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        argstr='--voxdump %d %d %d',
        desc='dump voxel GLM and exit')
    seed = traits.Int(argstr='--seed %d', desc='used for synthesizing noise')
    synth = traits.Bool(argstr='--synth', desc='replace input with gaussian')
    resynth_test = traits.Int(
        argstr='--resynthtest %d', desc='test GLM by resynthsis')
    profile = traits.Int(argstr='--profile %d', desc='niters : test speed')
    force_perm = traits.Bool(
        argstr='--perm-force',
        desc='force perumtation test, even when design matrix is not orthog')
    diag = traits.Int(argstr='--diag %d', desc='Gdiag_no : set diagnositc level')
    diag_cluster = traits.Bool(
        argstr='--diag-cluster',
        desc='save sig volume and exit from first sim loop')
    debug = traits.Bool(argstr='--debug', desc='turn on debugging')
    check_opts = traits.Bool(
        argstr='--checkopts',
        desc="don't run anything, just check options and exit")
    allow_repeated_subjects = traits.Bool(
        argstr='--allowsubjrep',
        desc=
        'allow subject names to repeat in the fsgd file (must appear before --fsgd'
    )
    allow_ill_cond = traits.Bool(
        argstr='--illcond', desc='allow ill-conditioned design matrices')
    sim_done_file = File(
        argstr='--sim-done %s', desc='create file when simulation finished')


class GLMFitOutputSpec(TraitedSpec):

    glm_dir = Directory(exists=True, desc="output directory")
    beta_file = File(exists=True, desc="map of regression coefficients")
    error_file = File(desc="map of residual error")
    error_var_file = File(desc="map of residual error variance")
    error_stddev_file = File(desc="map of residual error standard deviation")
    estimate_file = File(desc="map of the estimated Y values")
    mask_file = File(desc="map of the mask used in the analysis")
    fwhm_file = File(desc="text file with estimated smoothness")
    dof_file = File(
        desc="text file with effective degrees-of-freedom for the analysis")
    gamma_file = OutputMultiPath(
        desc="map of contrast of regression coefficients")
    gamma_var_file = OutputMultiPath(
        desc="map of regression contrast variance")
    sig_file = OutputMultiPath(desc="map of F-test significance (in -log10p)")
    ftest_file = OutputMultiPath(desc="map of test statistic values")
    spatial_eigenvectors = File(
        desc="map of spatial eigenvectors from residual PCA")
    frame_eigenvectors = File(
        desc="matrix of frame eigenvectors from residual PCA")
    singular_values = File(desc="matrix singular values from residual PCA")
    svd_stats_file = File(desc="text file summarizing the residual PCA")


class GLMFit(FSCommand):
    """Use FreeSurfer's mri_glmfit to specify and estimate a general linear model.

    Examples
    --------

    >>> glmfit = GLMFit()
    >>> glmfit.inputs.in_file = 'functional.nii'
    >>> glmfit.inputs.one_sample = True
    >>> glmfit.cmdline == 'mri_glmfit --glmdir %s --y functional.nii --osgm'%os.getcwd()
    True

    """

    _cmd = 'mri_glmfit'
    input_spec = GLMFitInputSpec
    output_spec = GLMFitOutputSpec

    def _format_arg(self, name, spec, value):
        if name == "surf":
            _si = self.inputs
            return spec.argstr % (_si.subject_id, _si.hemi, _si.surf_geo)
        return super(GLMFit, self)._format_arg(name, spec, value)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        # Get the top-level output directory
        if not isdefined(self.inputs.glm_dir):
            glmdir = os.getcwd()
        else:
            glmdir = os.path.abspath(self.inputs.glm_dir)
        outputs["glm_dir"] = glmdir

        # Assign the output files that always get created
        outputs["beta_file"] = os.path.join(glmdir, "beta.mgh")
        outputs["error_var_file"] = os.path.join(glmdir, "rvar.mgh")
        outputs["error_stddev_file"] = os.path.join(glmdir, "rstd.mgh")
        outputs["mask_file"] = os.path.join(glmdir, "mask.mgh")
        outputs["fwhm_file"] = os.path.join(glmdir, "fwhm.dat")
        outputs["dof_file"] = os.path.join(glmdir, "dof.dat")
        # Assign the conditional outputs
        if isdefined(self.inputs.save_residual) and self.inputs.save_residual:
            outputs["error_file"] = os.path.join(glmdir, "eres.mgh")
        if isdefined(self.inputs.save_estimate) and self.inputs.save_estimate:
            outputs["estimate_file"] = os.path.join(glmdir, "yhat.mgh")

        # Get the contrast directory name(s)
        if isdefined(self.inputs.contrast):
            contrasts = []
            for c in self.inputs.contrast:
                if split_filename(c)[2] in [".mat", ".dat", ".mtx", ".con"]:
                    contrasts.append(split_filename(c)[1])
                else:
                    contrasts.append(os.path.split(c)[1])
        elif isdefined(self.inputs.one_sample) and self.inputs.one_sample:
            contrasts = ["osgm"]

        # Add in the contrast images
        outputs["sig_file"] = [
            os.path.join(glmdir, c, "sig.mgh") for c in contrasts
        ]
        outputs["ftest_file"] = [
            os.path.join(glmdir, c, "F.mgh") for c in contrasts
        ]
        outputs["gamma_file"] = [
            os.path.join(glmdir, c, "gamma.mgh") for c in contrasts
        ]
        outputs["gamma_var_file"] = [
            os.path.join(glmdir, c, "gammavar.mgh") for c in contrasts
        ]

        # Add in the PCA results, if relevant
        if isdefined(self.inputs.pca) and self.inputs.pca:
            pcadir = os.path.join(glmdir, "pca-eres")
            outputs["spatial_eigenvectors"] = os.path.join(pcadir, "v.mgh")
            outputs["frame_eigenvectors"] = os.path.join(pcadir, "u.mtx")
            outputs["singluar_values"] = os.path.join(pcadir, "sdiag.mat")
            outputs["svd_stats_file"] = os.path.join(pcadir, "stats.dat")

        return outputs

    def _gen_filename(self, name):
        if name == 'glm_dir':
            return os.getcwd()
        return None


class OneSampleTTest(GLMFit):
    def __init__(self, **kwargs):
        super(OneSampleTTest, self).__init__(**kwargs)
        self.inputs.one_sample = True


class BinarizeInputSpec(FSTraitedSpec):
    in_file = File(
        exists=True,
        argstr='--i %s',
        mandatory=True,
        copyfile=False,
        desc='input volume')
    min = traits.Float(
        argstr='--min %f', xor=['wm_ven_csf'], desc='min thresh')
    max = traits.Float(
        argstr='--max %f', xor=['wm_ven_csf'], desc='max thresh')
    rmin = traits.Float(
        argstr='--rmin %f', desc='compute min based on rmin*globalmean')
    rmax = traits.Float(
        argstr='--rmax %f', desc='compute max based on rmax*globalmean')
    match = traits.List(
        traits.Int, argstr='--match %d...', desc='match instead of threshold')
    wm = traits.Bool(
        argstr='--wm',
        desc='set match vals to 2 and 41 (aseg for cerebral WM)')
    ventricles = traits.Bool(
        argstr='--ventricles',
        desc='set match vals those for aseg ventricles+choroid (not 4th)')
    wm_ven_csf = traits.Bool(
        argstr='--wm+vcsf',
        xor=['min', 'max'],
        desc='WM and ventricular CSF, including choroid (not 4th)')
    binary_file = File(
        argstr='--o %s', genfile=True, desc='binary output volume')
    out_type = traits.Enum(
        'nii', 'nii.gz', 'mgz', argstr='', desc='output file type')
    count_file = traits.Either(
        traits.Bool,
        File,
        argstr='--count %s',
        desc='save number of hits in ascii file (hits, ntotvox, pct)')
    bin_val = traits.Int(
        argstr='--binval %d',
        desc='set vox within thresh to val (default is 1)')
    bin_val_not = traits.Int(
        argstr='--binvalnot %d',
        desc='set vox outside range to val (default is 0)')
    invert = traits.Bool(argstr='--inv', desc='set binval=0, binvalnot=1')
    frame_no = traits.Int(
        argstr='--frame %s', desc='use 0-based frame of input (default is 0)')
    merge_file = File(
        exists=True, argstr='--merge %s', desc='merge with mergevol')
    mask_file = File(
        exists=True, argstr='--mask maskvol', desc='must be within mask')
    mask_thresh = traits.Float(
        argstr='--mask-thresh %f', desc='set thresh for mask')
    abs = traits.Bool(
        argstr='--abs', desc='take abs of invol first (ie, make unsigned)')
    bin_col_num = traits.Bool(
        argstr='--bincol',
        desc='set binarized voxel value to its column number')
    zero_edges = traits.Bool(
        argstr='--zero-edges', desc='zero the edge voxels')
    zero_slice_edge = traits.Bool(
        argstr='--zero-slice-edges', desc='zero the edge slice voxels')
    dilate = traits.Int(
        argstr='--dilate %d', desc='niters: dilate binarization in 3D')
    erode = traits.Int(
        argstr='--erode  %d',
        desc='nerode: erode binarization in 3D (after any dilation)')
    erode2d = traits.Int(
        argstr='--erode2d %d',
        desc='nerode2d: erode binarization in 2D (after any 3D erosion)')


class BinarizeOutputSpec(TraitedSpec):
    binary_file = File(exists=True, desc='binarized output volume')
    count_file = File(desc='ascii file containing number of hits')


class Binarize(FSCommand):
    """Use FreeSurfer mri_binarize to threshold an input volume

    Examples
    --------

    >>> binvol = Binarize(in_file='structural.nii', min=10, binary_file='foo_out.nii')
    >>> binvol.cmdline
    'mri_binarize --o foo_out.nii --i structural.nii --min 10.000000'

   """

    _cmd = 'mri_binarize'
    input_spec = BinarizeInputSpec
    output_spec = BinarizeOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outfile = self.inputs.binary_file
        if not isdefined(outfile):
            if isdefined(self.inputs.out_type):
                outfile = fname_presuffix(
                    self.inputs.in_file,
                    newpath=os.getcwd(),
                    suffix='.'.join(('_thresh', self.inputs.out_type)),
                    use_ext=False)
            else:
                outfile = fname_presuffix(
                    self.inputs.in_file, newpath=os.getcwd(), suffix='_thresh')
        outputs['binary_file'] = os.path.abspath(outfile)
        value = self.inputs.count_file
        if isdefined(value):
            if isinstance(value, bool):
                if value:
                    outputs['count_file'] = fname_presuffix(
                        self.inputs.in_file,
                        suffix='_count.txt',
                        newpath=os.getcwd(),
                        use_ext=False)
            else:
                outputs['count_file'] = value
        return outputs

    def _format_arg(self, name, spec, value):
        if name == 'count_file':
            if isinstance(value, bool):
                fname = self._list_outputs()[name]
            else:
                fname = value
            return spec.argstr % fname
        if name == 'out_type':
            return ''
        return super(Binarize, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        if name == 'binary_file':
            return self._list_outputs()[name]
        return None


class ConcatenateInputSpec(FSTraitedSpec):
    in_files = InputMultiPath(
        File(exists=True),
        desc='Individual volumes to be concatenated',
        argstr='--i %s...',
        mandatory=True)
    concatenated_file = File(
        desc='Output volume', argstr='--o %s', genfile=True)
    sign = traits.Enum(
        'abs',
        'pos',
        'neg',
        argstr='--%s',
        desc='Take only pos or neg voxles from input, or take abs')
    stats = traits.Enum(
        'sum',
        'var',
        'std',
        'max',
        'min',
        'mean',
        argstr='--%s',
        desc='Compute the sum, var, std, max, min or mean of the input volumes'
    )
    paired_stats = traits.Enum(
        'sum',
        'avg',
        'diff',
        'diff-norm',
        'diff-norm1',
        'diff-norm2',
        argstr='--paired-%s',
        desc='Compute paired sum, avg, or diff')
    gmean = traits.Int(
        argstr='--gmean %d',
        desc='create matrix to average Ng groups, Nper=Ntot/Ng')
    mean_div_n = traits.Bool(
        argstr='--mean-div-n', desc='compute mean/nframes (good for var)')
    multiply_by = traits.Float(
        argstr='--mul %f', desc='Multiply input volume by some amount')
    add_val = traits.Float(
        argstr='--add %f', desc='Add some amount to the input volume')
    multiply_matrix_file = File(
        exists=True,
        argstr='--mtx %s',
        desc='Multiply input by an ascii matrix in file')
    combine = traits.Bool(
        argstr='--combine',
        desc='Combine non-zero values into single frame volume')
    keep_dtype = traits.Bool(
        argstr='--keep-datatype',
        desc='Keep voxelwise precision type (default is float')
    max_bonfcor = traits.Bool(
        argstr='--max-bonfcor',
        desc='Compute max and bonferroni correct (assumes -log10(ps))')
    max_index = traits.Bool(
        argstr='--max-index',
        desc='Compute the index of max voxel in concatenated volumes')
    mask_file = File(
        exists=True, argstr='--mask %s', desc='Mask input with a volume')
    vote = traits.Bool(
        argstr='--vote',
        desc='Most frequent value at each voxel and fraction of occurances')
    sort = traits.Bool(
        argstr='--sort', desc='Sort each voxel by ascending frame value')


class ConcatenateOutputSpec(TraitedSpec):
    concatenated_file = File(
        exists=True, desc='Path/name of the output volume')


class Concatenate(FSCommand):
    """Use Freesurfer mri_concat to combine several input volumes
    into one output volume.  Can concatenate by frames, or compute
    a variety of statistics on the input volumes.

    Examples
    --------

    Combine two input volumes into one volume with two frames

    >>> concat = Concatenate()
    >>> concat.inputs.in_files = ['cont1.nii', 'cont2.nii']
    >>> concat.inputs.concatenated_file = 'bar.nii'
    >>> concat.cmdline
    'mri_concat --o bar.nii --i cont1.nii --i cont2.nii'

    """

    _cmd = 'mri_concat'
    input_spec = ConcatenateInputSpec
    output_spec = ConcatenateOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()

        fname = self.inputs.concatenated_file
        if not isdefined(fname):
            fname = 'concat_output.nii.gz'
        outputs['concatenated_file'] = os.path.join(os.getcwd(), fname)
        return outputs

    def _gen_filename(self, name):
        if name == 'concatenated_file':
            return self._list_outputs()[name]
        return None


class SegStatsInputSpec(FSTraitedSpec):
    _xor_inputs = ('segmentation_file', 'annot', 'surf_label')
    segmentation_file = File(
        exists=True,
        argstr='--seg %s',
        xor=_xor_inputs,
        mandatory=True,
        desc='segmentation volume path')
    annot = traits.Tuple(
        traits.Str,
        traits.Enum('lh', 'rh'),
        traits.Str,
        argstr='--annot %s %s %s',
        xor=_xor_inputs,
        mandatory=True,
        desc='subject hemi parc : use surface parcellation')
    surf_label = traits.Tuple(
        traits.Str,
        traits.Enum('lh', 'rh'),
        traits.Str,
        argstr='--slabel %s %s %s',
        xor=_xor_inputs,
        mandatory=True,
        desc='subject hemi label : use surface label')
    summary_file = File(
        argstr='--sum %s',
        genfile=True,
        position=-1,
        desc='Segmentation stats summary table file')
    partial_volume_file = File(
        exists=True, argstr='--pv %s', desc='Compensate for partial voluming')
    in_file = File(
        exists=True,
        argstr='--i %s',
        desc='Use the segmentation to report stats on this volume')
    frame = traits.Int(
        argstr='--frame %d', desc='Report stats on nth frame of input volume')
    multiply = traits.Float(argstr='--mul %f', desc='multiply input by val')
    calc_snr = traits.Bool(
        argstr='--snr', desc='save mean/std as extra column in output table')
    calc_power = traits.Enum(
        'sqr',
        'sqrt',
        argstr='--%s',
        desc='Compute either the sqr or the sqrt of the input')
    _ctab_inputs = ('color_table_file', 'default_color_table',
                    'gca_color_table')
    color_table_file = File(
        exists=True,
        argstr='--ctab %s',
        xor=_ctab_inputs,
        desc='color table file with seg id names')
    default_color_table = traits.Bool(
        argstr='--ctab-default',
        xor=_ctab_inputs,
        desc='use $FREESURFER_HOME/FreeSurferColorLUT.txt')
    gca_color_table = File(
        exists=True,
        argstr='--ctab-gca %s',
        xor=_ctab_inputs,
        desc='get color table from GCA (CMA)')
    segment_id = traits.List(
        argstr='--id %s...', desc='Manually specify segmentation ids')
    exclude_id = traits.Int(
        argstr='--excludeid %d', desc='Exclude seg id from report')
    exclude_ctx_gm_wm = traits.Bool(
        argstr='--excl-ctxgmwm', desc='exclude cortical gray and white matter')
    wm_vol_from_surf = traits.Bool(
        argstr='--surf-wm-vol', desc='Compute wm volume from surf')
    cortex_vol_from_surf = traits.Bool(
        argstr='--surf-ctx-vol', desc='Compute cortex volume from surf')
    non_empty_only = traits.Bool(
        argstr='--nonempty', desc='Only report nonempty segmentations')
    empty = traits.Bool(
        argstr="--empty",
        desc="Report on segmentations listed in the color table")
    mask_file = File(
        exists=True, argstr='--mask %s', desc='Mask volume (same size as seg')
    mask_thresh = traits.Float(
        argstr='--maskthresh %f',
        desc='binarize mask with this threshold <0.5>')
    mask_sign = traits.Enum(
        'abs',
        'pos',
        'neg',
        '--masksign %s',
        desc='Sign for mask threshold: pos, neg, or abs')
    mask_frame = traits.Int(
        '--maskframe %d',
        requires=['mask_file'],
        desc='Mask with this (0 based) frame of the mask volume')
    mask_invert = traits.Bool(
        argstr='--maskinvert', desc='Invert binarized mask volume')
    mask_erode = traits.Int(
        argstr='--maskerode %d', desc='Erode mask by some amount')
    brain_vol = traits.Enum(
        'brain-vol-from-seg',
        'brainmask',
        argstr='--%s',
        desc=
        'Compute brain volume either with ``brainmask`` or ``brain-vol-from-seg``'
    )
    brainmask_file = File(
        argstr="--brainmask %s",
        exists=True,
        desc=
        "Load brain mask and compute the volume of the brain as the non-zero voxels in this volume"
    )
    etiv = traits.Bool(
        argstr='--etiv', desc='Compute ICV from talairach transform')
    etiv_only = traits.Enum(
        'etiv',
        'old-etiv',
        '--%s-only',
        desc='Compute etiv and exit.  Use ``etiv`` or ``old-etiv``')
    avgwf_txt_file = traits.Either(
        traits.Bool,
        File,
        argstr='--avgwf %s',
        desc='Save average waveform into file (bool or filename)')
    avgwf_file = traits.Either(
        traits.Bool,
        File,
        argstr='--avgwfvol %s',
        desc='Save as binary volume (bool or filename)')
    sf_avg_file = traits.Either(
        traits.Bool,
        File,
        argstr='--sfavg %s',
        desc='Save mean across space and time')
    vox = traits.List(
        traits.Int,
        argstr='--vox %s',
        desc='Replace seg with all 0s except at C R S (three int inputs)')
    supratent = traits.Bool(
        argstr="--supratent", desc="Undocumented input flag")
    subcort_gm = traits.Bool(
        argstr="--subcortgray",
        desc="Compute volume of subcortical gray matter")
    total_gray = traits.Bool(
        argstr="--totalgray", desc="Compute volume of total gray matter")
    euler = traits.Bool(
        argstr="--euler",
        desc=
        "Write out number of defect holes in orig.nofix based on the euler number"
    )
    in_intensity = File(
        argstr="--in %s --in-intensity-name %s",
        desc="Undocumented input norm.mgz file")
    intensity_units = traits.Enum(
        'MR',
        argstr="--in-intensity-units %s",
        requires=["in_intensity"],
        desc="Intensity units")


class SegStatsOutputSpec(TraitedSpec):
    summary_file = File(
        exists=True, desc='Segmentation summary statistics table')
    avgwf_txt_file = File(
        desc='Text file with functional statistics averaged over segs')
    avgwf_file = File(
        desc='Volume with functional statistics averaged over segs')
    sf_avg_file = File(
        desc='Text file with func statistics averaged over segs and framss')


class SegStats(FSCommand):
    """Use FreeSurfer mri_segstats for ROI analysis

    Examples
    --------

    >>> import nipype.interfaces.freesurfer as fs
    >>> ss = fs.SegStats()
    >>> ss.inputs.annot = ('PWS04', 'lh', 'aparc')
    >>> ss.inputs.in_file = 'functional.nii'
    >>> ss.inputs.subjects_dir = '.'
    >>> ss.inputs.avgwf_txt_file = 'avgwf.txt'
    >>> ss.inputs.summary_file = 'summary.stats'
    >>> ss.cmdline
    'mri_segstats --annot PWS04 lh aparc --avgwf ./avgwf.txt --i functional.nii --sum ./summary.stats'

    """

    _cmd = 'mri_segstats'
    input_spec = SegStatsInputSpec
    output_spec = SegStatsOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.summary_file):
            outputs['summary_file'] = os.path.abspath(self.inputs.summary_file)
        else:
            outputs['summary_file'] = os.path.join(os.getcwd(),
                                                   'summary.stats')
        suffices = dict(
            avgwf_txt_file='_avgwf.txt',
            avgwf_file='_avgwf.nii.gz',
            sf_avg_file='sfavg.txt')
        if isdefined(self.inputs.segmentation_file):
            _, src = os.path.split(self.inputs.segmentation_file)
        if isdefined(self.inputs.annot):
            src = '_'.join(self.inputs.annot)
        if isdefined(self.inputs.surf_label):
            src = '_'.join(self.inputs.surf_label)
        for name, suffix in list(suffices.items()):
            value = getattr(self.inputs, name)
            if isdefined(value):
                if isinstance(value, bool):
                    outputs[name] = fname_presuffix(
                        src, suffix=suffix, newpath=os.getcwd(), use_ext=False)
                else:
                    outputs[name] = os.path.abspath(value)
        return outputs

    def _format_arg(self, name, spec, value):
        if name in ('summary_file', 'avgwf_txt_file'):
            if not isinstance(value, bool):
                if not os.path.isabs(value):
                    value = os.path.join('.', value)
        if name in ['avgwf_txt_file', 'avgwf_file', 'sf_avg_file']:
            if isinstance(value, bool):
                fname = self._list_outputs()[name]
            else:
                fname = value
            return spec.argstr % fname
        elif name == 'in_intensity':
            intensity_name = os.path.basename(
                self.inputs.in_intensity).replace('.mgz', '')
            return spec.argstr % (value, intensity_name)
        return super(SegStats, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        if name == 'summary_file':
            return self._list_outputs()[name]
        return None


class SegStatsReconAllInputSpec(SegStatsInputSpec):
    # recon-all input requirements
    subject_id = traits.String(
        'subject_id',
        usedefault=True,
        argstr="--subject %s",
        mandatory=True,
        desc="Subject id being processed")
    # implicit
    ribbon = File(
        mandatory=True, exists=True, desc="Input file mri/ribbon.mgz")
    presurf_seg = File(exists=True, desc="Input segmentation volume")
    transform = File(mandatory=True, exists=True, desc="Input transform file")
    lh_orig_nofix = File(
        mandatory=True, exists=True, desc="Input lh.orig.nofix")
    rh_orig_nofix = File(
        mandatory=True, exists=True, desc="Input rh.orig.nofix")
    lh_white = File(
        mandatory=True,
        exists=True,
        desc="Input file must be <subject_id>/surf/lh.white")
    rh_white = File(
        mandatory=True,
        exists=True,
        desc="Input file must be <subject_id>/surf/rh.white")
    lh_pial = File(
        mandatory=True,
        exists=True,
        desc="Input file must be <subject_id>/surf/lh.pial")
    rh_pial = File(
        mandatory=True,
        exists=True,
        desc="Input file must be <subject_id>/surf/rh.pial")
    aseg = File(exists=True, desc="Mandatory implicit input in 5.3")
    copy_inputs = traits.Bool(desc="If running as a node, set this to True " +
                              "otherwise, this will copy the implicit inputs "
                              + "to the node directory.")


class SegStatsReconAll(SegStats):
    """
    This class inherits SegStats and modifies it for use in a recon-all workflow.
    This implementation mandates implicit inputs that SegStats.
    To ensure backwards compatability of SegStats, this class was created.

    Examples
    ========
    >>> from nipype.interfaces.freesurfer import SegStatsReconAll
    >>> segstatsreconall = SegStatsReconAll()
    >>> segstatsreconall.inputs.annot = ('PWS04', 'lh', 'aparc')
    >>> segstatsreconall.inputs.avgwf_txt_file = 'avgwf.txt'
    >>> segstatsreconall.inputs.summary_file = 'summary.stats'
    >>> segstatsreconall.inputs.subject_id = '10335'
    >>> segstatsreconall.inputs.ribbon = 'wm.mgz'
    >>> segstatsreconall.inputs.transform = 'trans.mat'
    >>> segstatsreconall.inputs.presurf_seg = 'wm.mgz'
    >>> segstatsreconall.inputs.lh_orig_nofix = 'lh.pial'
    >>> segstatsreconall.inputs.rh_orig_nofix = 'lh.pial'
    >>> segstatsreconall.inputs.lh_pial = 'lh.pial'
    >>> segstatsreconall.inputs.rh_pial = 'lh.pial'
    >>> segstatsreconall.inputs.lh_white = 'lh.pial'
    >>> segstatsreconall.inputs.rh_white = 'lh.pial'
    >>> segstatsreconall.inputs.empty = True
    >>> segstatsreconall.inputs.brain_vol = 'brain-vol-from-seg'
    >>> segstatsreconall.inputs.exclude_ctx_gm_wm = True
    >>> segstatsreconall.inputs.supratent = True
    >>> segstatsreconall.inputs.subcort_gm = True
    >>> segstatsreconall.inputs.etiv = True
    >>> segstatsreconall.inputs.wm_vol_from_surf = True
    >>> segstatsreconall.inputs.cortex_vol_from_surf = True
    >>> segstatsreconall.inputs.total_gray = True
    >>> segstatsreconall.inputs.euler = True
    >>> segstatsreconall.inputs.exclude_id = 0
    >>> segstatsreconall.cmdline
    'mri_segstats --annot PWS04 lh aparc --avgwf ./avgwf.txt --brain-vol-from-seg --surf-ctx-vol --empty --etiv --euler --excl-ctxgmwm --excludeid 0 --subcortgray --subject 10335 --supratent --totalgray --surf-wm-vol --sum ./summary.stats'
    """
    input_spec = SegStatsReconAllInputSpec
    output_spec = SegStatsOutputSpec

    def _format_arg(self, name, spec, value):
        if name == 'brainmask_file':
            return spec.argstr % os.path.basename(value)
        return super(SegStatsReconAll, self)._format_arg(name, spec, value)

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if 'subjects_dir' in inputs:
                inputs['subjects_dir'] = self.inputs.subjects_dir
            copy2subjdir(self, self.inputs.lh_orig_nofix, 'surf',
                         'lh.orig.nofix')
            copy2subjdir(self, self.inputs.rh_orig_nofix, 'surf',
                         'rh.orig.nofix')
            copy2subjdir(self, self.inputs.lh_white, 'surf', 'lh.white')
            copy2subjdir(self, self.inputs.rh_white, 'surf', 'rh.white')
            copy2subjdir(self, self.inputs.lh_pial, 'surf', 'lh.pial')
            copy2subjdir(self, self.inputs.rh_pial, 'surf', 'rh.pial')
            copy2subjdir(self, self.inputs.ribbon, 'mri', 'ribbon.mgz')
            copy2subjdir(self, self.inputs.presurf_seg, 'mri',
                         'aseg.presurf.mgz')
            copy2subjdir(self, self.inputs.aseg, 'mri', 'aseg.mgz')
            copy2subjdir(self, self.inputs.transform,
                         os.path.join('mri', 'transforms'), 'talairach.xfm')
            copy2subjdir(self, self.inputs.in_intensity, 'mri')
            copy2subjdir(self, self.inputs.brainmask_file, 'mri')
        return super(SegStatsReconAll, self).run(**inputs)


class Label2VolInputSpec(FSTraitedSpec):
    label_file = InputMultiPath(
        File(exists=True),
        argstr='--label %s...',
        xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        copyfile=False,
        mandatory=True,
        desc='list of label files')
    annot_file = File(
        exists=True,
        argstr='--annot %s',
        xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        requires=('subject_id', 'hemi'),
        mandatory=True,
        copyfile=False,
        desc='surface annotation file')
    seg_file = File(
        exists=True,
        argstr='--seg %s',
        xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        mandatory=True,
        copyfile=False,
        desc='segmentation file')
    aparc_aseg = traits.Bool(
        argstr='--aparc+aseg',
        xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        mandatory=True,
        desc='use aparc+aseg.mgz in subjectdir as seg')
    template_file = File(
        exists=True,
        argstr='--temp %s',
        mandatory=True,
        desc='output template volume')
    reg_file = File(
        exists=True,
        argstr='--reg %s',
        xor=('reg_file', 'reg_header', 'identity'),
        desc='tkregister style matrix VolXYZ = R*LabelXYZ')
    reg_header = File(
        exists=True,
        argstr='--regheader %s',
        xor=('reg_file', 'reg_header', 'identity'),
        desc='label template volume')
    identity = traits.Bool(
        argstr='--identity',
        xor=('reg_file', 'reg_header', 'identity'),
        desc='set R=I')
    invert_mtx = traits.Bool(
        argstr='--invertmtx', desc='Invert the registration matrix')
    fill_thresh = traits.Range(
        0., 1., argstr='--fillthresh %g', desc='thresh : between 0 and 1')
    label_voxel_volume = traits.Float(
        argstr='--labvoxvol %f', desc='volume of each label point (def 1mm3)')
    proj = traits.Tuple(
        traits.Enum('abs', 'frac'),
        traits.Float,
        traits.Float,
        traits.Float,
        argstr='--proj %s %f %f %f',
        requires=('subject_id', 'hemi'),
        desc='project along surface normal')
    subject_id = traits.Str(argstr='--subject %s', desc='subject id')
    hemi = traits.Enum(
        'lh', 'rh', argstr='--hemi %s', desc='hemisphere to use lh or rh')
    surface = traits.Str(
        argstr='--surf %s', desc='use surface instead of white')
    vol_label_file = File(argstr='--o %s', genfile=True, desc='output volume')
    label_hit_file = File(
        argstr='--hits %s', desc='file with each frame is nhits for a label')
    map_label_stat = File(
        argstr='--label-stat %s',
        desc='map the label stats field into the vol')
    native_vox2ras = traits.Bool(
        argstr='--native-vox2ras',
        desc='use native vox2ras xform instead of  tkregister-style')


class Label2VolOutputSpec(TraitedSpec):
    vol_label_file = File(exists=True, desc='output volume')


class Label2Vol(FSCommand):
    """Make a binary volume from a Freesurfer label

    Examples
    --------

    >>> binvol = Label2Vol(label_file='cortex.label', template_file='structural.nii', reg_file='register.dat', fill_thresh=0.5, vol_label_file='foo_out.nii')
    >>> binvol.cmdline
    'mri_label2vol --fillthresh 0.5 --label cortex.label --reg register.dat --temp structural.nii --o foo_out.nii'

   """

    _cmd = 'mri_label2vol'
    input_spec = Label2VolInputSpec
    output_spec = Label2VolOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outfile = self.inputs.vol_label_file
        if not isdefined(outfile):
            for key in ['label_file', 'annot_file', 'seg_file']:
                if isdefined(getattr(self.inputs, key)):
                    path = getattr(self.inputs, key)
                    if isinstance(path, list):
                        path = path[0]
                    _, src = os.path.split(path)
            if isdefined(self.inputs.aparc_aseg):
                src = 'aparc+aseg.mgz'
            outfile = fname_presuffix(
                src, suffix='_vol.nii.gz', newpath=os.getcwd(), use_ext=False)
        outputs['vol_label_file'] = outfile
        return outputs

    def _gen_filename(self, name):
        if name == 'vol_label_file':
            return self._list_outputs()[name]
        return None


class MS_LDAInputSpec(FSTraitedSpec):
    lda_labels = traits.List(
        traits.Int(),
        argstr='-lda %s',
        mandatory=True,
        minlen=2,
        maxlen=2,
        sep=' ',
        desc='pair of class labels to optimize')
    weight_file = File(
        argstr='-weight %s',
        mandatory=True,
        desc='filename for the LDA weights (input or output)')
    vol_synth_file = File(
        exists=False,
        argstr='-synth %s',
        mandatory=True,
        desc=('filename for the synthesized output '
              'volume'))
    label_file = File(
        exists=True, argstr='-label %s', desc='filename of the label volume')
    mask_file = File(
        exists=True,
        argstr='-mask %s',
        desc='filename of the brain mask volume')
    shift = traits.Int(
        argstr='-shift %d',
        desc='shift all values equal to the given value to zero')
    conform = traits.Bool(
        argstr='-conform',
        desc=('Conform the input volumes (brain mask '
              'typically already conformed)'))
    use_weights = traits.Bool(
        argstr='-W',
        desc=('Use the weights from a previously '
              'generated weight file'))
    images = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        copyfile=False,
        desc='list of input FLASH images',
        position=-1)


class MS_LDAOutputSpec(TraitedSpec):
    weight_file = File(exists=True, desc='')
    vol_synth_file = File(exists=True, desc='')


class MS_LDA(FSCommand):
    """Perform LDA reduction on the intensity space of an arbitrary # of FLASH images

    Examples
    --------

    >>> grey_label = 2
    >>> white_label = 3
    >>> zero_value = 1
    >>> optimalWeights = MS_LDA(lda_labels=[grey_label, white_label], \
                                label_file='label.mgz', weight_file='weights.txt', \
                                shift=zero_value, vol_synth_file='synth_out.mgz', \
                                conform=True, use_weights=True, \
                                images=['FLASH1.mgz', 'FLASH2.mgz', 'FLASH3.mgz'])
    >>> optimalWeights.cmdline
    'mri_ms_LDA -conform -label label.mgz -lda 2 3 -shift 1 -W -synth synth_out.mgz -weight weights.txt FLASH1.mgz FLASH2.mgz FLASH3.mgz'
    """

    _cmd = 'mri_ms_LDA'
    input_spec = MS_LDAInputSpec
    output_spec = MS_LDAOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.output_synth):
            outputs['vol_synth_file'] = os.path.abspath(
                self.inputs.output_synth)
        else:
            outputs['vol_synth_file'] = os.path.abspath(
                self.inputs.vol_synth_file)
        if not isdefined(
                self.inputs.use_weights) or self.inputs.use_weights is False:
            outputs['weight_file'] = os.path.abspath(self.inputs.weight_file)
        return outputs

    def _verify_weights_file_exists(self):
        if not os.path.exists(os.path.abspath(self.inputs.weight_file)):
            raise traits.TraitError(
                "MS_LDA: use_weights must accompany an existing weights file")

    def _format_arg(self, name, spec, value):
        if name == 'use_weights':
            if self.inputs.use_weights is True:
                self._verify_weights_file_exists()
            else:
                return ''
                # TODO: Fix bug when boolean values are set explicitly to false
        return super(MS_LDA, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        pass


class Label2LabelInputSpec(FSTraitedSpec):
    hemisphere = traits.Enum(
        'lh',
        'rh',
        argstr="--hemi %s",
        mandatory=True,
        desc="Input hemisphere")
    subject_id = traits.String(
        'subject_id',
        usedefault=True,
        argstr="--trgsubject %s",
        mandatory=True,
        desc="Target subject")
    sphere_reg = File(
        mandatory=True,
        exists=True,
        desc="Implicit input <hemisphere>.sphere.reg")
    white = File(
        mandatory=True, exists=True, desc="Implicit input <hemisphere>.white")
    source_sphere_reg = File(
        mandatory=True,
        exists=True,
        desc="Implicit input <hemisphere>.sphere.reg")
    source_white = File(
        mandatory=True, exists=True, desc="Implicit input <hemisphere>.white")
    source_label = File(
        argstr="--srclabel %s",
        mandatory=True,
        exists=True,
        desc="Source label")
    source_subject = traits.String(
        argstr="--srcsubject %s", mandatory=True, desc="Source subject name")
    # optional
    out_file = File(
        argstr="--trglabel %s",
        name_source=['source_label'],
        name_template='%s_converted',
        hash_files=False,
        keep_extension=True,
        desc="Target label")
    registration_method = traits.Enum(
        'surface',
        'volume',
        usedefault=True,
        argstr="--regmethod %s",
        desc="Registration method")
    copy_inputs = traits.Bool(
        desc="If running as a node, set this to True." +
        "This will copy the input files to the node " + "directory.")


class Label2LabelOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='Output label')


class Label2Label(FSCommand):
    """
    Converts a label in one subject's space to a label
    in another subject's space using either talairach or spherical
    as an intermediate registration space.

    If a source mask is used, then the input label must have been
    created from a surface (ie, the vertex numbers are valid). The
    format can be anything supported by mri_convert or curv or paint.
    Vertices in the source label that do not meet threshold in the
    mask will be removed from the label.

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import Label2Label
    >>> l2l = Label2Label()
    >>> l2l.inputs.hemisphere = 'lh'
    >>> l2l.inputs.subject_id = '10335'
    >>> l2l.inputs.sphere_reg = 'lh.pial'
    >>> l2l.inputs.white = 'lh.pial'
    >>> l2l.inputs.source_subject = 'fsaverage'
    >>> l2l.inputs.source_label = 'lh-pial.stl'
    >>> l2l.inputs.source_white = 'lh.pial'
    >>> l2l.inputs.source_sphere_reg = 'lh.pial'
    >>> l2l.cmdline
    'mri_label2label --hemi lh --trglabel lh-pial_converted.stl --regmethod surface --srclabel lh-pial.stl --srcsubject fsaverage --trgsubject 10335'
    """

    _cmd = 'mri_label2label'
    input_spec = Label2LabelInputSpec
    output_spec = Label2LabelOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_file'] = os.path.join(self.inputs.subjects_dir,
                                           self.inputs.subject_id, 'label',
                                           self.inputs.out_file)
        return outputs

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if 'subjects_dir' in inputs:
                inputs['subjects_dir'] = self.inputs.subjects_dir
            hemi = self.inputs.hemisphere
            copy2subjdir(self, self.inputs.sphere_reg, 'surf',
                         '{0}.sphere.reg'.format(hemi))
            copy2subjdir(self, self.inputs.white, 'surf',
                         '{0}.white'.format(hemi))
            copy2subjdir(
                self,
                self.inputs.source_sphere_reg,
                'surf',
                '{0}.sphere.reg'.format(hemi),
                subject_id=self.inputs.source_subject)
            copy2subjdir(
                self,
                self.inputs.source_white,
                'surf',
                '{0}.white'.format(hemi),
                subject_id=self.inputs.source_subject)

        # label dir must exist in order for output file to be written
        label_dir = os.path.join(self.inputs.subjects_dir,
                                 self.inputs.subject_id, 'label')
        if not os.path.isdir(label_dir):
            os.makedirs(label_dir)

        return super(Label2Label, self).run(**inputs)


class Label2AnnotInputSpec(FSTraitedSpec):
    # required
    hemisphere = traits.Enum(
        'lh',
        'rh',
        argstr="--hemi %s",
        mandatory=True,
        desc="Input hemisphere")
    subject_id = traits.String(
        'subject_id',
        usedefault=True,
        argstr="--s %s",
        mandatory=True,
        desc="Subject name/ID")
    in_labels = traits.List(
        argstr="--l %s...", mandatory=True, desc="List of input label files")
    out_annot = traits.String(
        argstr="--a %s",
        mandatory=True,
        desc="Name of the annotation to create")
    orig = File(exists=True, mandatory=True, desc="implicit {hemisphere}.orig")
    # optional
    keep_max = traits.Bool(
        argstr="--maxstatwinner", desc="Keep label with highest 'stat' value")
    verbose_off = traits.Bool(
        argstr="--noverbose",
        desc="Turn off overlap and stat override messages")
    color_table = File(
        argstr="--ctab %s",
        exists=True,
        desc=
        "File that defines the structure names, their indices, and their color"
    )
    copy_inputs = traits.Bool(
        desc="copy implicit inputs and create a temp subjects_dir")


class Label2AnnotOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='Output annotation file')


class Label2Annot(FSCommand):
    """
    Converts a set of surface labels to an annotation file

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import Label2Annot
    >>> l2a = Label2Annot()
    >>> l2a.inputs.hemisphere = 'lh'
    >>> l2a.inputs.subject_id = '10335'
    >>> l2a.inputs.in_labels = ['lh.aparc.label']
    >>> l2a.inputs.orig = 'lh.pial'
    >>> l2a.inputs.out_annot = 'test'
    >>> l2a.cmdline
    'mris_label2annot --hemi lh --l lh.aparc.label --a test --s 10335'
    """

    _cmd = 'mris_label2annot'
    input_spec = Label2AnnotInputSpec
    output_spec = Label2AnnotOutputSpec

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if 'subjects_dir' in inputs:
                inputs['subjects_dir'] = self.inputs.subjects_dir
            copy2subjdir(
                self,
                self.inputs.orig,
                folder='surf',
                basename='{0}.orig'.format(self.inputs.hemisphere))
        # label dir must exist in order for output file to be written
        label_dir = os.path.join(self.inputs.subjects_dir,
                                 self.inputs.subject_id, 'label')
        if not os.path.isdir(label_dir):
            os.makedirs(label_dir)
        return super(Label2Annot, self).run(**inputs)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_file"] = os.path.join(
            str(self.inputs.subjects_dir), str(self.inputs.subject_id),
            'label',
            str(self.inputs.hemisphere) + '.' + str(self.inputs.out_annot) +
            '.annot')
        return outputs


class SphericalAverageInputSpec(FSTraitedSpec):
    out_file = File(
        argstr="%s",
        genfile=True,
        exists=False,
        position=-1,
        desc="Output filename")
    in_average = Directory(
        argstr="%s",
        exists=True,
        genfile=True,
        position=-2,
        desc="Average subject")
    in_surf = File(
        argstr="%s",
        mandatory=True,
        exists=True,
        position=-3,
        desc="Input surface file")
    hemisphere = traits.Enum(
        'lh',
        'rh',
        argstr="%s",
        mandatory=True,
        position=-4,
        desc="Input hemisphere")
    fname = traits.String(
        argstr="%s",
        mandatory=True,
        position=-5,
        desc="""Filename from the average subject directory.
                          Example: to use rh.entorhinal.label as the input label
                          filename, set fname to 'rh.entorhinal' and which to
                          'label'. The program will then search for
                          '{in_average}/label/rh.entorhinal.label'
                          """)
    which = traits.Enum(
        'coords',
        'label',
        'vals',
        'curv',
        'area',
        argstr="%s",
        mandatory=True,
        position=-6,
        desc="No documentation")
    subject_id = traits.String(
        argstr="-o %s", mandatory=True, desc="Output subject id")
    # optional
    erode = traits.Int(argstr="-erode %d", desc="Undocumented")
    in_orig = File(
        argstr="-orig %s", exists=True, desc="Original surface filename")
    threshold = traits.Float(argstr="-t %.1f", desc="Undocumented")


class SphericalAverageOutputSpec(TraitedSpec):
    out_file = File(exists=False, desc='Output label')


class SphericalAverage(FSCommand):
    """
    This program will add a template into an average surface.

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import SphericalAverage
    >>> sphericalavg = SphericalAverage()
    >>> sphericalavg.inputs.out_file = 'test.out'
    >>> sphericalavg.inputs.in_average = '.'
    >>> sphericalavg.inputs.in_surf = 'lh.pial'
    >>> sphericalavg.inputs.hemisphere = 'lh'
    >>> sphericalavg.inputs.fname = 'lh.entorhinal'
    >>> sphericalavg.inputs.which = 'label'
    >>> sphericalavg.inputs.subject_id = '10335'
    >>> sphericalavg.inputs.erode = 2
    >>> sphericalavg.inputs.threshold = 5
    >>> sphericalavg.cmdline
    'mris_spherical_average -erode 2 -o 10335 -t 5.0 label lh.entorhinal lh pial . test.out'
    """

    _cmd = 'mris_spherical_average'
    input_spec = SphericalAverageInputSpec
    output_spec = SphericalAverageOutputSpec

    def _format_arg(self, name, spec, value):
        if name == 'in_orig' or name == 'in_surf':
            surf = os.path.basename(value)
            for item in ['lh.', 'rh.']:
                surf = surf.replace(item, '')
            return spec.argstr % surf
        return super(SphericalAverage, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        if name == 'in_average':
            avg_subject = str(self.inputs.hemisphere) + '.EC_average'
            avg_directory = os.path.join(self.inputs.subjects_dir, avg_subject)
            if not os.path.isdir(avg_directory):
                fs_home = os.path.abspath(os.environ.get('FREESURFER_HOME'))
            return avg_subject
        elif name == 'out_file':
            return self._list_outputs()[name]
        else:
            return None

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.out_file):
            outputs['out_file'] = os.path.abspath(self.inputs.out_file)
        else:
            out_dir = os.path.join(self.inputs.subjects_dir,
                                   self.inputs.subject_id, 'label')
            if isdefined(self.inputs.in_average):
                basename = os.path.basename(self.inputs.in_average)
                basename = basename.replace('_', '_exvivo_') + '.label'
            else:
                basename = str(
                    self.inputs.hemisphere) + '.EC_exvivo_average.label'
            outputs['out_file'] = os.path.join(out_dir, basename)
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import MELODIC


def test_MELODIC_inputs():
    input_map = dict(
        ICs=dict(
            argstr='--ICs=%s',
            extensions=None,
        ),
        approach=dict(argstr='-a %s', ),
        args=dict(argstr='%s', ),
        bg_image=dict(
            argstr='--bgimage=%s',
            extensions=None,
        ),
        bg_threshold=dict(argstr='--bgthreshold=%f', ),
        cov_weight=dict(argstr='--covarweight=%f', ),
        dim=dict(argstr='-d %d', ),
        dim_est=dict(argstr='--dimest=%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epsilon=dict(argstr='--eps=%f', ),
        epsilonS=dict(argstr='--epsS=%f', ),
        in_files=dict(
            argstr='-i %s',
            mandatory=True,
            position=0,
            sep=',',
        ),
        log_power=dict(argstr='--logPower', ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        max_restart=dict(argstr='--maxrestart=%d', ),
        maxit=dict(argstr='--maxit=%d', ),
        migp=dict(argstr='--migp', ),
        migpN=dict(argstr='--migpN %d', ),
        migp_factor=dict(argstr='--migp_factor %d', ),
        migp_shuffle=dict(argstr='--migp_shuffle', ),
        mix=dict(
            argstr='--mix=%s',
            extensions=None,
        ),
        mm_thresh=dict(argstr='--mmthresh=%f', ),
        no_bet=dict(argstr='--nobet', ),
        no_mask=dict(argstr='--nomask', ),
        no_mm=dict(argstr='--no_mm', ),
        non_linearity=dict(argstr='--nl=%s', ),
        num_ICs=dict(argstr='-n %d', ),
        out_all=dict(argstr='--Oall', ),
        out_dir=dict(
            argstr='-o %s',
            genfile=True,
        ),
        out_mean=dict(argstr='--Omean', ),
        out_orig=dict(argstr='--Oorig', ),
        out_pca=dict(argstr='--Opca', ),
        out_stats=dict(argstr='--Ostats', ),
        out_unmix=dict(argstr='--Ounmix', ),
        out_white=dict(argstr='--Owhite', ),
        output_type=dict(),
        pbsc=dict(argstr='--pbsc', ),
        rem_cmp=dict(argstr='-f %d', ),
        remove_deriv=dict(argstr='--remove_deriv', ),
        report=dict(argstr='--report', ),
        report_maps=dict(argstr='--report_maps=%s', ),
        s_con=dict(
            argstr='--Scon=%s',
            extensions=None,
        ),
        s_des=dict(
            argstr='--Sdes=%s',
            extensions=None,
        ),
        sep_vn=dict(argstr='--sep_vn', ),
        sep_whiten=dict(argstr='--sep_whiten', ),
        smode=dict(
            argstr='--smode=%s',
            extensions=None,
        ),
        t_con=dict(
            argstr='--Tcon=%s',
            extensions=None,
        ),
        t_des=dict(
            argstr='--Tdes=%s',
            extensions=None,
        ),
        tr_sec=dict(argstr='--tr=%f', ),
        update_mask=dict(argstr='--update_mask', ),
        var_norm=dict(argstr='--vn', ),
    )
    inputs = MELODIC.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MELODIC_outputs():
    output_map = dict(
        out_dir=dict(),
        report_dir=dict(),
    )
    outputs = MELODIC.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class N4ITKBiasFieldCorrectionInputSpec(CommandLineInputSpec):
    inputimage = File(
        desc="Input image where you observe signal inhomegeneity",
        exists=True,
        argstr="--inputimage %s")
    maskimage = File(
        desc=
        "Binary mask that defines the structure of your interest. NOTE: This parameter is OPTIONAL. If the mask is not specified, the module will use internally Otsu thresholding to define this mask. Better processing results can often be obtained when a meaningful mask is defined.",
        exists=True,
        argstr="--maskimage %s")
    outputimage = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Result of processing",
        argstr="--outputimage %s")
    outputbiasfield = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Recovered bias field (OPTIONAL)",
        argstr="--outputbiasfield %s")
    iterations = InputMultiPath(
        traits.Int,
        desc=
        "Maximum number of iterations at each level of resolution. Larger values will increase execution time, but may lead to better results.",
        sep=",",
        argstr="--iterations %s")
    convergencethreshold = traits.Float(
        desc=
        "Stopping criterion for the iterative bias estimation. Larger values will lead to smaller execution time.",
        argstr="--convergencethreshold %f")
    meshresolution = InputMultiPath(
        traits.Float,
        desc=
        "Resolution of the initial bspline grid defined as a sequence of three numbers. The actual resolution will be defined by adding the bspline order (default is 3) to the resolution in each dimension specified here. For example, 1,1,1 will result in a 4x4x4 grid of control points. This parameter may need to be adjusted based on your input image. In the multi-resolution N4 framework, the resolution of the bspline grid at subsequent iterations will be doubled. The number of resolutions is implicitly defined by Number of iterations parameter (the size of this list is the number of resolutions)",
        sep=",",
        argstr="--meshresolution %s")
    splinedistance = traits.Float(
        desc=
        "An alternative means to define the spline grid, by setting the distance between the control points. This parameter is used only if the grid resolution is not specified.",
        argstr="--splinedistance %f")
    shrinkfactor = traits.Int(
        desc=
        "Defines how much the image should be upsampled before estimating the inhomogeneity field. Increase if you want to reduce the execution time. 1 corresponds to the original resolution. Larger values will significantly reduce the computation time.",
        argstr="--shrinkfactor %d")
    bsplineorder = traits.Int(
        desc=
        "Order of B-spline used in the approximation. Larger values will lead to longer execution times, may result in overfitting and poor result.",
        argstr="--bsplineorder %d")
    weightimage = File(
        desc="Weight Image", exists=True, argstr="--weightimage %s")
    histogramsharpening = InputMultiPath(
        traits.Float,
        desc=
        "A vector of up to three values. Non-zero values correspond to Bias Field Full Width at Half Maximum, Wiener filter noise, and Number of histogram bins.",
        sep=",",
        argstr="--histogramsharpening %s")


class N4ITKBiasFieldCorrectionOutputSpec(TraitedSpec):
    outputimage = File(desc="Result of processing", exists=True)
    outputbiasfield = File(desc="Recovered bias field (OPTIONAL)", exists=True)


class N4ITKBiasFieldCorrection(SEMLikeCommandLine):
    """title: N4ITK MRI Bias correction

category: Filtering

description: Performs image bias correction using N4 algorithm. This module is based on the ITK filters contributed in the following publication:  Tustison N, Gee J "N4ITK: Nick's N3 ITK Implementation For MRI Bias Field Correction", The Insight Journal 2009 January-June, http://hdl.handle.net/10380/3053

version: 9

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/N4ITKBiasFieldCorrection

contributor: Nick Tustison (UPenn), Andrey Fedorov (SPL, BWH), Ron Kikinis (SPL, BWH)

acknowledgements: The development of this module was partially supported by NIH grants R01 AA016748-01, R01 CA111288 and U01 CA151261 as well as by NA-MIC, NAC, NCIGT and the Slicer community.

"""

    input_spec = N4ITKBiasFieldCorrectionInputSpec
    output_spec = N4ITKBiasFieldCorrectionOutputSpec
    _cmd = "N4ITKBiasFieldCorrection "
    _outputs_filenames = {
        'outputimage': 'outputimage.nii',
        'outputbiasfield': 'outputbiasfield.nii'
    }



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..confounds import ACompCor


def test_ACompCor_inputs():
    input_map = dict(
        components_file=dict(usedefault=True, ),
        failure_mode=dict(usedefault=True, ),
        header_prefix=dict(),
        high_pass_cutoff=dict(usedefault=True, ),
        ignore_initial_volumes=dict(usedefault=True, ),
        mask_files=dict(),
        mask_index=dict(
            requires=['mask_files'],
            xor=['merge_method'],
        ),
        mask_names=dict(),
        merge_method=dict(
            requires=['mask_files'],
            xor=['mask_index'],
        ),
        num_components=dict(xor=['variance_threshold'], ),
        pre_filter=dict(usedefault=True, ),
        realigned_file=dict(
            extensions=None,
            mandatory=True,
        ),
        regress_poly_degree=dict(usedefault=True, ),
        repetition_time=dict(),
        save_metadata=dict(usedefault=True, ),
        save_pre_filter=dict(usedefault=True, ),
        use_regress_poly=dict(
            deprecated='0.15.0',
            new_name='pre_filter',
        ),
        variance_threshold=dict(xor=['num_components'], ),
    )
    inputs = ACompCor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ACompCor_outputs():
    output_map = dict(
        components_file=dict(extensions=None, ),
        metadata_file=dict(extensions=None, ),
        pre_filter_file=dict(extensions=None, ),
    )
    outputs = ACompCor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import SynthesizeFLASH


def test_SynthesizeFLASH_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_weighting=dict(
            argstr='-w',
            position=1,
        ),
        flip_angle=dict(
            argstr='%.2f',
            mandatory=True,
            position=3,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
        ),
        pd_image=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=6,
        ),
        subjects_dir=dict(),
        t1_image=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=5,
        ),
        te=dict(
            argstr='%.3f',
            mandatory=True,
            position=4,
        ),
        tr=dict(
            argstr='%.2f',
            mandatory=True,
            position=2,
        ),
    )
    inputs = SynthesizeFLASH.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SynthesizeFLASH_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = SynthesizeFLASH.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import pytest

from ....testing import example_data
from ...niftyreg import get_custom_path

from ..asl import FitAsl
from ...niftyreg.tests.test_regutils import no_nifty_tool


@pytest.mark.skipif(
    no_nifty_tool(cmd='fit_asl'), reason="niftyfit is not installed")
def test_fit_asl():
    """ Testing FitAsl interface."""
    # Create the test node
    fit_asl = FitAsl()

    # Check if the command is properly defined
    cmd = get_custom_path('fit_asl', env_dir='NIFTYFIT_DIR')
    assert fit_asl.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        fit_asl.run()

    # Tests on the interface:
    # Runs cbf fitting assuming all tissue is GM!
    in_file = example_data('asl.nii.gz')
    fit_asl.inputs.source_file = in_file

    cmd_tmp = '{cmd} -source {in_file} -cbf {cbf} -error {error} -syn {syn}'
    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        cbf='asl_cbf.nii.gz',
        error='asl_error.nii.gz',
        syn='asl_syn.nii.gz',
    )

    assert fit_asl.cmdline == expected_cmd

    # Runs cbf fitting using IR/SR T1 data to estimate the local T1 and uses
    # the segmentation data to fit tissue specific blood flow parameters
    # (lambda,transit times,T1)
    fit_asl2 = FitAsl(sig=True)
    in_file = example_data('asl.nii.gz')
    t1map = example_data('T1map.nii.gz')
    seg = example_data('segmentation0.nii.gz')
    fit_asl2.inputs.source_file = in_file
    fit_asl2.inputs.t1map = t1map
    fit_asl2.inputs.seg = seg

    cmd_tmp = '{cmd} -source {in_file} -cbf {cbf} -error {error} \
-seg {seg} -sig -syn {syn} -t1map {t1map}'

    expected_cmd = cmd_tmp.format(
        cmd=cmd,
        in_file=in_file,
        t1map=t1map,
        seg=seg,
        cbf='asl_cbf.nii.gz',
        error='asl_error.nii.gz',
        syn='asl_syn.nii.gz',
    )

    assert fit_asl2.cmdline == expected_cmd



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import unicode_literals
from builtins import str, bytes

import os
import numpy as np

import pytest
from nipype.testing.fixtures import create_files_in_directory

import nipype.interfaces.spm.base as spm
from nipype.interfaces.spm import no_spm
import nipype.interfaces.matlab as mlab
from nipype.interfaces.spm.base import SPMCommandInputSpec
from nipype.interfaces.base import traits

mlab.MatlabCommand.set_default_matlab_cmd(os.getenv('MATLABCMD', 'matlab'))


def test_scan_for_fnames(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    names = spm.scans_for_fnames(filelist, keep4d=True)
    assert names[0] == filelist[0]
    assert names[1] == filelist[1]


save_time = False
if not save_time:

    @pytest.mark.skipif(no_spm(), reason="spm is not installed")
    def test_spm_path():
        spm_path = spm.Info.path()
        if spm_path is not None:
            assert isinstance(spm_path, (str, bytes))
            assert 'spm' in spm_path.lower()


def test_use_mfile():
    class TestClass(spm.SPMCommand):
        input_spec = spm.SPMCommandInputSpec

    dc = TestClass()  # dc = derived_class
    assert dc.inputs.mfile


def test_find_mlab_cmd_defaults():
    saved_env = dict(os.environ)

    class TestClass(spm.SPMCommand):
        pass

    # test without FORCE_SPMMCR, SPMMCRCMD set
    for varname in ['FORCE_SPMMCR', 'SPMMCRCMD']:
        try:
            del os.environ[varname]
        except KeyError:
            pass
    dc = TestClass()
    assert dc._use_mcr is None
    assert dc._matlab_cmd is None
    # test with only FORCE_SPMMCR set
    os.environ['FORCE_SPMMCR'] = '1'
    dc = TestClass()
    assert dc._use_mcr
    assert dc._matlab_cmd is None
    # test with both, FORCE_SPMMCR and SPMMCRCMD set
    os.environ['SPMMCRCMD'] = 'spmcmd'
    dc = TestClass()
    assert dc._use_mcr
    assert dc._matlab_cmd == 'spmcmd'
    # restore environment
    os.environ.clear()
    os.environ.update(saved_env)


@pytest.mark.skipif(no_spm(), reason="spm is not installed")
def test_cmd_update():
    class TestClass(spm.SPMCommand):
        input_spec = spm.SPMCommandInputSpec

    dc = TestClass()  # dc = derived_class
    dc.inputs.matlab_cmd = 'foo'
    assert dc.mlab._cmd == 'foo'


def test_cmd_update2():
    class TestClass(spm.SPMCommand):
        _jobtype = 'jobtype'
        _jobname = 'jobname'
        input_spec = spm.SPMCommandInputSpec

    dc = TestClass()  # dc = derived_class
    assert dc.jobtype == 'jobtype'
    assert dc.jobname == 'jobname'


def test_reformat_dict_for_savemat():
    class TestClass(spm.SPMCommand):
        input_spec = spm.SPMCommandInputSpec

    dc = TestClass()  # dc = derived_class
    out = dc._reformat_dict_for_savemat({'a': {'b': {'c': []}}})
    assert out == [{'a': [{'b': [{'c': []}]}]}]


def test_generate_job(create_files_in_directory):
    class TestClass(spm.SPMCommand):
        input_spec = spm.SPMCommandInputSpec

    dc = TestClass()  # dc = derived_class
    out = dc._generate_job()
    assert out == ''
    # struct array
    contents = {'contents': [1, 2, 3, 4]}
    out = dc._generate_job(contents=contents)
    assert out == ('.contents(1) = 1;\n.contents(2) = 2;'
                   '\n.contents(3) = 3;\n.contents(4) = 4;\n')
    # cell array of strings
    filelist, outdir = create_files_in_directory
    names = spm.scans_for_fnames(filelist, keep4d=True)
    contents = {'files': names}
    out = dc._generate_job(prefix='test', contents=contents)
    assert out == "test.files = {...\n'a.nii';...\n'b.nii';...\n};\n"
    # string assignment
    contents = 'foo'
    out = dc._generate_job(prefix='test', contents=contents)
    assert out == "test = 'foo';\n"
    # cell array of vectors
    contents = {'onsets': np.array((1, ), dtype=object)}
    contents['onsets'][0] = [1, 2, 3, 4]
    out = dc._generate_job(prefix='test', contents=contents)
    assert out == 'test.onsets = {...\n[1, 2, 3, 4];...\n};\n'


def test_bool():
    class TestClassInputSpec(SPMCommandInputSpec):
        test_in = include_intercept = traits.Bool(field='testfield')

    class TestClass(spm.SPMCommand):
        input_spec = TestClassInputSpec
        _jobtype = 'jobtype'
        _jobname = 'jobname'

    dc = TestClass()  # dc = derived_class
    dc.inputs.test_in = True
    out = dc._make_matlab_command(dc._parse_inputs())
    assert out.find('jobs{1}.spm.jobtype.jobname.testfield = 1;') > 0, 1
    dc.inputs.use_v8struct = False
    out = dc._make_matlab_command(dc._parse_inputs())
    assert out.find('jobs{1}.jobtype{1}.jobname{1}.testfield = 1;') > 0, 1


def test_make_matlab_command(create_files_in_directory):
    class TestClass(spm.SPMCommand):
        _jobtype = 'jobtype'
        _jobname = 'jobname'
        input_spec = spm.SPMCommandInputSpec

    dc = TestClass()  # dc = derived_class
    filelist, outdir = create_files_in_directory
    contents = {'contents': [1, 2, 3, 4]}
    script = dc._make_matlab_command([contents])
    assert 'jobs{1}.spm.jobtype.jobname.contents(3) = 3;' in script
    dc.inputs.use_v8struct = False
    script = dc._make_matlab_command([contents])
    assert 'jobs{1}.jobtype{1}.jobname{1}.contents(3) = 3;' in script



import pytest
from collections import namedtuple
from ...base import traits, File, TraitedSpec, BaseInterfaceInputSpec
from ..base import (convert_to_traits_type, create_interface_specs,
                    dipy_to_nipype_interface, DipyBaseInterface, no_dipy,
                    get_dipy_workflows)


def test_convert_to_traits_type():
    Params = namedtuple("Params", "traits_type is_file")
    Res = namedtuple("Res", "traits_type is_mandatory")
    l_entries = [Params('variable string', False),
                 Params('variable int', False),
                 Params('variable float', False),
                 Params('variable bool', False),
                 Params('variable complex', False),
                 Params('variable int, optional', False),
                 Params('variable string, optional', False),
                 Params('variable float, optional', False),
                 Params('variable bool, optional', False),
                 Params('variable complex, optional', False),
                 Params('string', False), Params('int', False),
                 Params('string', True), Params('float', False),
                 Params('bool', False), Params('complex', False),
                 Params('string, optional', False),
                 Params('int, optional', False),
                 Params('string, optional', True),
                 Params('float, optional', False),
                 Params('bool, optional', False),
                 Params('complex, optional', False),
                 ]
    l_expected = [Res(traits.ListStr, True), Res(traits.ListInt, True),
                  Res(traits.ListFloat, True), Res(traits.ListBool, True),
                  Res(traits.ListComplex, True), Res(traits.ListInt, False),
                  Res(traits.ListStr, False), Res(traits.ListFloat, False),
                  Res(traits.ListBool, False), Res(traits.ListComplex, False),
                  Res(traits.Str, True), Res(traits.Int, True),
                  Res(File, True), Res(traits.Float, True),
                  Res(traits.Bool, True), Res(traits.Complex, True),
                  Res(traits.Str, False), Res(traits.Int, False),
                  Res(File, False), Res(traits.Float, False),
                  Res(traits.Bool, False), Res(traits.Complex, False),
                  ]

    for entry, res in zip(l_entries, l_expected):
        traits_type, is_mandatory = convert_to_traits_type(entry.traits_type,
                                                           entry.is_file)
        assert traits_type == res.traits_type
        assert is_mandatory == res.is_mandatory

    with pytest.raises(IOError):
        convert_to_traits_type("file, optional")


def test_create_interface_specs():
    new_interface = create_interface_specs("MyInterface")

    assert new_interface.__base__ == TraitedSpec
    assert isinstance(new_interface(), TraitedSpec)
    assert new_interface.__name__ == "MyInterface"
    assert not new_interface().get()

    new_interface = create_interface_specs("MyInterface",
                                           BaseClass=BaseInterfaceInputSpec)
    assert new_interface.__base__ == BaseInterfaceInputSpec
    assert isinstance(new_interface(), BaseInterfaceInputSpec)
    assert new_interface.__name__ == "MyInterface"
    assert not new_interface().get()

    params = [("params1", "string", ["my description"]), ("params2_files", "string", ["my description @"]),
              ("params3", "int, optional", ["useful option"]), ("out_params", "string", ["my out description"])]

    new_interface = create_interface_specs("MyInterface", params=params,
                                           BaseClass=BaseInterfaceInputSpec)

    assert new_interface.__base__ == BaseInterfaceInputSpec
    assert isinstance(new_interface(), BaseInterfaceInputSpec)
    assert new_interface.__name__ == "MyInterface"
    current_params = new_interface().get()
    assert len(current_params) == 4
    assert 'params1' in current_params.keys()
    assert 'params2_files' in current_params.keys()
    assert 'params3' in current_params.keys()
    assert 'out_params' in current_params.keys()


@pytest.mark.skipif(no_dipy(), reason="DIPY is not installed")
def test_dipy_to_nipype_interface():
    from dipy.workflows.workflow import Workflow

    class DummyWorkflow(Workflow):

        @classmethod
        def get_short_name(cls):
            return 'dwf1'

        def run(self, in_files, param1=1, out_dir='', out_ref='out1.txt'):
            """Workflow used to test basic workflows.

            Parameters
            ----------
            in_files : string
                fake input string param
            param1 : int, optional
                fake positional param (default 1)
            out_dir : string, optional
                fake output directory (default '')
            out_ref : string, optional
                fake out file (default out1.txt)

            References
            -----------
            dummy references

            """
            return param1

    new_specs = dipy_to_nipype_interface("MyModelSpec", DummyWorkflow)
    assert new_specs.__base__ == DipyBaseInterface
    assert isinstance(new_specs(), DipyBaseInterface)
    assert new_specs.__name__ == "MyModelSpec"
    assert hasattr(new_specs, 'input_spec')
    assert new_specs().input_spec.__base__ == BaseInterfaceInputSpec
    assert hasattr(new_specs, 'output_spec')
    assert new_specs().output_spec.__base__ == TraitedSpec
    assert hasattr(new_specs, '_run_interface')
    assert hasattr(new_specs, '_list_outputs')
    params_in = new_specs().inputs.get()
    params_out = new_specs()._outputs().get()
    assert len(params_in) == 4
    assert 'in_files' in params_in.keys()
    assert 'param1' in params_in.keys()
    assert 'out_dir' in params_out.keys()
    assert 'out_ref' in params_out.keys()

    with pytest.raises(ValueError):
        new_specs().run()


@pytest.mark.skipif(no_dipy(), reason="DIPY is not installed")
def test_get_dipy_workflows():
    from dipy.workflows import align

    l_wkflw = get_dipy_workflows(align)
    for name, obj in l_wkflw:
        assert name.endswith('Flow')
        assert issubclass(obj, align.Workflow)


if __name__ == "__main__":
    test_convert_to_traits_type()
    test_create_interface_specs()
    test_dipy_to_nipype_interface()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import AntsJointFusion


def test_AntsJointFusion_inputs():
    input_map = dict(
        alpha=dict(
            argstr='-a %s',
            usedefault=True,
        ),
        args=dict(argstr='%s', ),
        atlas_image=dict(
            argstr='-g %s...',
            mandatory=True,
        ),
        atlas_segmentation_image=dict(
            argstr='-l %s...',
            mandatory=True,
        ),
        beta=dict(
            argstr='-b %s',
            usedefault=True,
        ),
        constrain_nonnegative=dict(
            argstr='-c',
            usedefault=True,
        ),
        dimension=dict(argstr='-d %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exclusion_image=dict(),
        exclusion_image_label=dict(
            argstr='-e %s',
            requires=['exclusion_image'],
        ),
        mask_image=dict(
            argstr='-x %s',
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_atlas_voting_weight_name_format=dict(requires=[
            'out_label_fusion', 'out_intensity_fusion_name_format',
            'out_label_post_prob_name_format'
        ], ),
        out_intensity_fusion_name_format=dict(argstr='', ),
        out_label_fusion=dict(
            argstr='%s',
            extensions=None,
            hash_files=False,
        ),
        out_label_post_prob_name_format=dict(
            requires=['out_label_fusion',
                      'out_intensity_fusion_name_format'], ),
        patch_metric=dict(argstr='-m %s', ),
        patch_radius=dict(
            argstr='-p %s',
            maxlen=3,
            minlen=3,
        ),
        retain_atlas_voting_images=dict(
            argstr='-f',
            usedefault=True,
        ),
        retain_label_posterior_images=dict(
            argstr='-r',
            requires=['atlas_segmentation_image'],
            usedefault=True,
        ),
        search_radius=dict(
            argstr='-s %s',
            usedefault=True,
        ),
        target_image=dict(
            argstr='-t %s',
            mandatory=True,
        ),
        verbose=dict(argstr='-v', ),
    )
    inputs = AntsJointFusion.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AntsJointFusion_outputs():
    output_map = dict(
        out_atlas_voting_weight_name_format=dict(),
        out_intensity_fusion_name_format=dict(),
        out_label_fusion=dict(extensions=None, ),
        out_label_post_prob_name_format=dict(),
    )
    outputs = AntsJointFusion.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ImageStats


def test_ImageStats_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_files=dict(
            argstr='-images %s',
            mandatory=True,
            position=-1,
        ),
        out_type=dict(
            argstr='-outputdatatype %s',
            usedefault=True,
        ),
        output_root=dict(
            argstr='-outputroot %s',
            extensions=None,
            mandatory=True,
        ),
        stat=dict(
            argstr='-stat %s',
            mandatory=True,
            units='NA',
        ),
    )
    inputs = ImageStats.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ImageStats_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = ImageStats.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Provides interfaces to various commands provided by diffusion toolkit
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
from ..base import (TraitedSpec, File, traits, CommandLine, InputMultiPath,
                    CommandLineInputSpec)

__docformat__ = 'restructuredtext'


class SplineFilterInputSpec(CommandLineInputSpec):
    track_file = File(
        exists=True,
        desc="file containing tracks to be filtered",
        position=0,
        argstr="%s",
        mandatory=True)
    step_length = traits.Float(
        desc="in the unit of minimum voxel size",
        position=1,
        argstr="%f",
        mandatory=True)
    output_file = File(
        "spline_tracks.trk",
        desc="target file for smoothed tracks",
        position=2,
        argstr="%s",
        usedefault=True)


class SplineFilterOutputSpec(TraitedSpec):
    smoothed_track_file = File(exists=True)


class SplineFilter(CommandLine):
    """
    Smoothes TrackVis track files with a B-Spline filter.

    Helps remove redundant track points and segments
    (thus reducing the size of the track file) and also
    make tracks nicely smoothed. It will NOT change the
    quality of the tracks or lose any original information.

    Example
    -------

    >>> import nipype.interfaces.diffusion_toolkit as dtk
    >>> filt = dtk.SplineFilter()
    >>> filt.inputs.track_file = 'tracks.trk'
    >>> filt.inputs.step_length = 0.5
    >>> filt.run()                                 # doctest: +SKIP
    """
    input_spec = SplineFilterInputSpec
    output_spec = SplineFilterOutputSpec

    _cmd = "spline_filter"

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['smoothed_track_file'] = os.path.abspath(
            self.inputs.output_file)
        return outputs


class TrackMergeInputSpec(CommandLineInputSpec):
    track_files = InputMultiPath(
        File(exists=True),
        desc="file containing tracks to be filtered",
        position=0,
        argstr="%s...",
        mandatory=True)
    output_file = File(
        "merged_tracks.trk",
        desc="target file for merged tracks",
        position=-1,
        argstr="%s",
        usedefault=True)


class TrackMergeOutputSpec(TraitedSpec):
    track_file = File(exists=True)


class TrackMerge(CommandLine):
    """
    Merges several TrackVis track files into a single track
    file.

    An id type property tag is added to each track in the
    newly merged file, with each unique id representing where
    the track was originally from. When the merged file is
    loaded in TrackVis, a property filter will show up in
    Track Property panel. Users can adjust that to distinguish
    and sub-group tracks by its id (origin).

    Example
    -------

    >>> import nipype.interfaces.diffusion_toolkit as dtk
    >>> mrg = dtk.TrackMerge()
    >>> mrg.inputs.track_files = ['track1.trk','track2.trk']
    >>> mrg.run()                                 # doctest: +SKIP
    """
    input_spec = TrackMergeInputSpec
    output_spec = TrackMergeOutputSpec

    _cmd = "track_merge"

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['track_file'] = os.path.abspath(self.inputs.output_file)
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import AnalyzeWarp


def test_AnalyzeWarp_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        jac=dict(
            argstr='-jac %s',
            usedefault=True,
        ),
        jacmat=dict(
            argstr='-jacmat %s',
            usedefault=True,
        ),
        moving_image=dict(
            argstr='-in %s',
            extensions=None,
        ),
        num_threads=dict(
            argstr='-threads %01d',
            nohash=True,
            usedefault=True,
        ),
        output_path=dict(
            argstr='-out %s',
            mandatory=True,
            usedefault=True,
        ),
        points=dict(
            argstr='-def %s',
            position=0,
            usedefault=True,
        ),
        transform_file=dict(
            argstr='-tp %s',
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = AnalyzeWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AnalyzeWarp_outputs():
    output_map = dict(
        disp_field=dict(extensions=None, ),
        jacdet_map=dict(extensions=None, ),
        jacmat_map=dict(extensions=None, ),
    )
    outputs = AnalyzeWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..c3 import C3d


def test_C3d_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            mandatory=True,
            position=1,
        ),
        interp=dict(argstr='-interpolation %s', ),
        is_4d=dict(usedefault=True, ),
        multicomp_split=dict(
            argstr='-mcr',
            position=0,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            position=-1,
            xor=['out_files'],
        ),
        out_files=dict(
            argstr='-oo %s',
            position=-1,
            xor=['out_file'],
        ),
        pix_type=dict(argstr='-type %s', ),
        resample=dict(argstr='-resample %s', ),
        scale=dict(argstr='-scale %s', ),
        shift=dict(argstr='-shift %s', ),
        smooth=dict(argstr='-smooth %s', ),
    )
    inputs = C3d.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_C3d_outputs():
    output_map = dict(out_files=dict(), )
    outputs = C3d.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import division
from builtins import range
from nipype.testing import example_data
from nipype.interfaces.base import InputMultiPath
from traits.trait_errors import TraitError
from nipype.interfaces.ants import JointFusion
import pytest


def test_JointFusion_dimension():
    at = JointFusion()
    set_dimension = lambda d: setattr(at.inputs, 'dimension', int(d))
    for d in range(2, 5):
        set_dimension(d)
        assert at.inputs.dimension == int(d)
    for d in [0, 1, 6, 7]:
        with pytest.raises(TraitError):
            set_dimension(d)


@pytest.mark.parametrize("m", range(1, 5))
def test_JointFusion_modalities(m):
    at = JointFusion()
    setattr(at.inputs, 'modalities', int(m))
    assert at.inputs.modalities == int(m)


@pytest.mark.parametrize("a, b",
                         [(a, b) for a in range(10) for b in range(10)])
def test_JointFusion_method(a, b):
    at = JointFusion()
    set_method = lambda a, b: setattr(at.inputs, 'method', 'Joint[%.1f,%d]'.format(a, b))
    _a = a / 10.0
    set_method(_a, b)
    # set directly
    assert at.inputs.method == 'Joint[%.1f,%d]'.format(_a, b)
    aprime = _a + 0.1
    bprime = b + 1
    at.inputs.alpha = aprime
    at.inputs.beta = bprime
    # set with alpha/beta
    assert at.inputs.method == 'Joint[%.1f,%d]'.format(aprime, bprime)


@pytest.mark.parametrize("attr, x",
                         [(attr, x)
                          for attr in ['patch_radius', 'search_radius']
                          for x in range(5)])
def test_JointFusion_radius(attr, x):
    at = JointFusion()
    setattr(at.inputs, attr, [x, x + 1, x**x])
    assert at._format_arg(attr, None, getattr(
        at.inputs, attr))[4:] == '{0}x{1}x{2}'.format(x, x + 1, x**x)


def test_JointFusion_cmd():
    at = JointFusion()
    at.inputs.dimension = 3
    at.inputs.modalities = 1
    at.inputs.method = 'Joint[0.1,2]'
    at.inputs.output_label_image = 'fusion_labelimage_output.nii'
    warped_intensity_images = [
        example_data('im1.nii'),
        example_data('im2.nii')
    ]
    at.inputs.warped_intensity_images = warped_intensity_images
    segmentation_images = [
        example_data('segmentation0.nii.gz'),
        example_data('segmentation1.nii.gz')
    ]
    at.inputs.warped_label_images = segmentation_images
    T1_image = example_data('T1.nii')
    at.inputs.target_image = T1_image
    at.inputs.patch_radius = [3, 2, 1]
    at.inputs.search_radius = [1, 2, 3]
    expected_command = ('jointfusion 3 1 -m Joint[0.1,2] -rp 3x2x1 -rs 1x2x3'
                        ' -tg %s -g %s -g %s -l %s -l %s'
                        ' fusion_labelimage_output.nii') % (
                            T1_image, warped_intensity_images[0],
                            warped_intensity_images[1], segmentation_images[0],
                            segmentation_images[1])
    assert at.cmdline == expected_command
    # setting intensity or labels with unequal lengths raises error
    with pytest.raises(AssertionError):
        at._format_arg('warped_intensity_images', InputMultiPath,
                       warped_intensity_images + [example_data('im3.nii')])



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Cerebro


def test_Cerebro_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        costFunction=dict(
            argstr='-c %d',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputAtlasLabelFile=dict(
            argstr='--atlaslabels %s',
            extensions=None,
            mandatory=True,
        ),
        inputAtlasMRIFile=dict(
            argstr='--atlas %s',
            extensions=None,
            mandatory=True,
        ),
        inputBrainMaskFile=dict(
            argstr='-m %s',
            extensions=None,
        ),
        inputMRIFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        keepTempFiles=dict(argstr='--keep', ),
        linearConvergence=dict(argstr='--linconv %f', ),
        outputAffineTransformFile=dict(
            argstr='--air %s',
            extensions=None,
            genfile=True,
        ),
        outputCerebrumMaskFile=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
        ),
        outputLabelVolumeFile=dict(
            argstr='-l %s',
            extensions=None,
            genfile=True,
        ),
        outputWarpTransformFile=dict(
            argstr='--warp %s',
            extensions=None,
            genfile=True,
        ),
        tempDirectory=dict(argstr='--tempdir %s', ),
        tempDirectoryBase=dict(argstr='--tempdirbase %s', ),
        useCentroids=dict(argstr='--centroids', ),
        verbosity=dict(argstr='-v %d', ),
        warpConvergence=dict(argstr='--warpconv %f', ),
        warpLabel=dict(argstr='--warplevel %d', ),
    )
    inputs = Cerebro.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Cerebro_outputs():
    output_map = dict(
        outputAffineTransformFile=dict(extensions=None, ),
        outputCerebrumMaskFile=dict(extensions=None, ),
        outputLabelVolumeFile=dict(extensions=None, ),
        outputWarpTransformFile=dict(extensions=None, ),
    )
    outputs = Cerebro.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



import os
import shutil
from tempfile import mkdtemp
import pytest
import numpy
import py.path as pp

NIPYPE_DATADIR = os.path.realpath(
    os.path.join(os.path.dirname(__file__), 'testing/data'))
temp_folder = mkdtemp()
data_dir = os.path.join(temp_folder, 'data')
shutil.copytree(NIPYPE_DATADIR, data_dir)


@pytest.fixture(autouse=True)
def add_np(doctest_namespace):
    doctest_namespace['np'] = numpy
    doctest_namespace['os'] = os
    doctest_namespace["datadir"] = data_dir


@pytest.fixture(autouse=True)
def _docdir(request):
    """Grabbed from https://stackoverflow.com/a/46991331"""
    # Trigger ONLY for the doctests.
    doctest_plugin = request.config.pluginmanager.getplugin("doctest")
    if isinstance(request.node, doctest_plugin.DoctestItem):

        # Get the fixture dynamically by its name.
        tmpdir = pp.local(data_dir)

        # Chdir only for the duration of the test.
        with tmpdir.as_cwd():
            yield

    else:
        # For normal tests, we have to yield, since this is a yield-fixture.
        yield


def pytest_unconfigure(config):
    # Delete temp folder after session is finished
    shutil.rmtree(temp_folder)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..convert import ProcStreamlines


def test_ProcStreamlines_inputs():
    input_map = dict(
        allowmultitargets=dict(argstr='-allowmultitargets', ),
        args=dict(argstr='%s', ),
        datadims=dict(
            argstr='-datadims %s',
            units='voxels',
        ),
        directional=dict(
            argstr='-directional %s',
            units='NA',
        ),
        discardloops=dict(argstr='-discardloops', ),
        endpointfile=dict(
            argstr='-endpointfile %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exclusionfile=dict(
            argstr='-exclusionfile %s',
            extensions=None,
        ),
        gzip=dict(argstr='-gzip', ),
        in_file=dict(
            argstr='-inputfile %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        inputmodel=dict(
            argstr='-inputmodel %s',
            usedefault=True,
        ),
        iterations=dict(
            argstr='-iterations %d',
            units='NA',
        ),
        maxtractlength=dict(
            argstr='-maxtractlength %d',
            units='mm',
        ),
        maxtractpoints=dict(
            argstr='-maxtractpoints %d',
            units='NA',
        ),
        mintractlength=dict(
            argstr='-mintractlength %d',
            units='mm',
        ),
        mintractpoints=dict(
            argstr='-mintractpoints %d',
            units='NA',
        ),
        noresample=dict(argstr='-noresample', ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        outputacm=dict(
            argstr='-outputacm',
            requires=['outputroot', 'seedfile'],
        ),
        outputcbs=dict(
            argstr='-outputcbs',
            requires=['outputroot', 'targetfile', 'seedfile'],
        ),
        outputcp=dict(
            argstr='-outputcp',
            requires=['outputroot', 'seedfile'],
        ),
        outputroot=dict(
            argstr='-outputroot %s',
            extensions=None,
        ),
        outputsc=dict(
            argstr='-outputsc',
            requires=['outputroot', 'seedfile'],
        ),
        outputtracts=dict(argstr='-outputtracts', ),
        regionindex=dict(
            argstr='-regionindex %d',
            units='mm',
        ),
        resamplestepsize=dict(
            argstr='-resamplestepsize %d',
            units='NA',
        ),
        seedfile=dict(
            argstr='-seedfile %s',
            extensions=None,
        ),
        seedpointmm=dict(
            argstr='-seedpointmm %s',
            units='mm',
        ),
        seedpointvox=dict(
            argstr='-seedpointvox %s',
            units='voxels',
        ),
        targetfile=dict(
            argstr='-targetfile %s',
            extensions=None,
        ),
        truncateinexclusion=dict(argstr='-truncateinexclusion', ),
        truncateloops=dict(argstr='-truncateloops', ),
        voxeldims=dict(
            argstr='-voxeldims %s',
            units='mm',
        ),
        waypointfile=dict(
            argstr='-waypointfile %s',
            extensions=None,
        ),
    )
    inputs = ProcStreamlines.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ProcStreamlines_outputs():
    output_map = dict(
        outputroot_files=dict(),
        proc=dict(extensions=None, ),
    )
    outputs = ProcStreamlines.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Seg


def test_Seg_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bias_classes=dict(argstr='-bias_classes %s', ),
        bias_fwhm=dict(argstr='-bias_fwhm %f', ),
        blur_meth=dict(argstr='-blur_meth %s', ),
        bmrf=dict(argstr='-bmrf %f', ),
        classes=dict(argstr='-classes %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-anat %s',
            copyfile=True,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        main_N=dict(argstr='-main_N %d', ),
        mask=dict(
            argstr='-mask %s',
            mandatory=True,
            position=-2,
        ),
        mixfloor=dict(argstr='-mixfloor %f', ),
        mixfrac=dict(argstr='-mixfrac %s', ),
        prefix=dict(argstr='-prefix %s', ),
    )
    inputs = Seg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Seg_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Seg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import VolumeMask


def test_VolumeMask_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        aseg=dict(
            extensions=None,
            xor=['in_aseg'],
        ),
        copy_inputs=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_aseg=dict(
            argstr='--aseg_name %s',
            extensions=None,
            xor=['aseg'],
        ),
        left_ribbonlabel=dict(
            argstr='--label_left_ribbon %d',
            mandatory=True,
        ),
        left_whitelabel=dict(
            argstr='--label_left_white %d',
            mandatory=True,
        ),
        lh_pial=dict(
            extensions=None,
            mandatory=True,
        ),
        lh_white=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_pial=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_white=dict(
            extensions=None,
            mandatory=True,
        ),
        right_ribbonlabel=dict(
            argstr='--label_right_ribbon %d',
            mandatory=True,
        ),
        right_whitelabel=dict(
            argstr='--label_right_white %d',
            mandatory=True,
        ),
        save_ribbon=dict(argstr='--save_ribbon', ),
        subject_id=dict(
            argstr='%s',
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        subjects_dir=dict(),
    )
    inputs = VolumeMask.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_VolumeMask_outputs():
    output_map = dict(
        lh_ribbon=dict(extensions=None, ),
        out_ribbon=dict(extensions=None, ),
        rh_ribbon=dict(extensions=None, ),
    )
    outputs = VolumeMask.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .segmentation import SimilarityIndex, BRAINSTalairach, BRAINSTalairachMask
from .utilities import (HistogramMatchingFilter, GenerateEdgeMapImage,
                        GeneratePurePlugMask)
from .classify import BRAINSPosteriorToContinuousClass



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..epi import EddyCorrect


def test_EddyCorrect_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_edc',
            output_name='eddy_corrected',
            position=1,
        ),
        output_type=dict(),
        ref_num=dict(
            argstr='%d',
            mandatory=True,
            position=2,
            usedefault=True,
        ),
    )
    inputs = EddyCorrect.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_EddyCorrect_outputs():
    output_map = dict(eddy_corrected=dict(extensions=None, ), )
    outputs = EddyCorrect.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""DTITK registration interfaces

DTI-TK developed by Gary Hui Zhang, gary.zhang@ucl.ac.uk
For additional help, visit http://dti-tk.sf.net

The high-dimensional tensor-based DTI registration algorithm

Zhang, H., Avants, B.B, Yushkevich, P.A., Woo, J.H., Wang, S., McCluskey, L.H.,
 Elman, L.B., Melhem, E.R., Gee, J.C., High-dimensional spatial normalization
 of diffusion tensor images improves the detection of white matter differences
 in amyotrophic lateral sclerosis, IEEE Transactions on Medical Imaging,
 26(11):1585-1597, November 2007. PMID: 18041273.

The original piecewise-affine tensor-based DTI registration algorithm at the
core of DTI-TK

Zhang, H., Yushkevich, P.A., Alexander, D.C., Gee, J.C., Deformable
 registration of diffusion tensor MR images with explicit orientation
 optimization, Medical Image Analysis, 10(5):764-785, October 2006. PMID:
 16899392.

"""

from ..base import TraitedSpec, CommandLineInputSpec, traits, File, isdefined
from ...utils.filemanip import fname_presuffix, split_filename
from .base import CommandLineDtitk, DTITKRenameMixin
import os

__docformat__ = 'restructuredtext'


class RigidInputSpec(CommandLineInputSpec):
    fixed_file = File(desc="fixed tensor volume", exists=True,
                      mandatory=True, position=0, argstr="%s", copyfile=False)
    moving_file = File(desc="moving tensor volume", exists=True,
                       mandatory=True, position=1, argstr="%s", copyfile=False)
    similarity_metric = traits.Enum('EDS', 'GDS', 'DDS', 'NMI',
                                    mandatory=True, position=2, argstr="%s",
                                    desc="similarity metric", usedefault=True)
    sampling_xyz = traits.Tuple((4, 4, 4), mandatory=True, position=3,
                                argstr="%g %g %g", usedefault=True,
                                desc="dist between samp points (mm) (x,y,z)")
    ftol = traits.Float(mandatory=True, position=4, argstr="%g",
                        desc="cost function tolerance", default_value=0.01,
                        usedefault=True)
    initialize_xfm = File(copyfile=True, desc="Initialize w/DTITK-FORMAT"
                          "affine", position=5, argstr="%s", exists=True)


class RigidOutputSpec(TraitedSpec):
    out_file = File(exists=True)
    out_file_xfm = File(exists=True)


class Rigid(CommandLineDtitk):
    """Performs rigid registration between two tensor volumes

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.Rigid()
    >>> node.inputs.fixed_file = 'im1.nii'
    >>> node.inputs.moving_file = 'im2.nii'
    >>> node.inputs.similarity_metric = 'EDS'
    >>> node.inputs.sampling_xyz = (4,4,4)
    >>> node.inputs.ftol = 0.01
    >>> node.cmdline
    'dti_rigid_reg im1.nii im2.nii EDS 4 4 4 0.01'
    >>> node.run() # doctest: +SKIP
    """
    input_spec = RigidInputSpec
    output_spec = RigidOutputSpec
    _cmd = 'dti_rigid_reg'

    '''def _format_arg(self, name, spec, value):
        if name == 'initialize_xfm':
            value = 1
        return super(Rigid, self)._format_arg(name, spec, value)'''

    def _run_interface(self, runtime):
        runtime = super(Rigid, self)._run_interface(runtime)
        if '''.aff doesn't exist or can't be opened''' in runtime.stderr:
            self.raise_exception(runtime)
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()
        moving = self.inputs.moving_file
        outputs['out_file_xfm'] = fname_presuffix(moving, suffix='.aff',
                                                  use_ext=False)
        outputs['out_file'] = fname_presuffix(moving, suffix='_aff')
        return outputs


class Affine(Rigid):
    """Performs affine registration between two tensor volumes

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.Affine()
    >>> node.inputs.fixed_file = 'im1.nii'
    >>> node.inputs.moving_file = 'im2.nii'
    >>> node.inputs.similarity_metric = 'EDS'
    >>> node.inputs.sampling_xyz = (4,4,4)
    >>> node.inputs.ftol = 0.01
    >>> node.inputs.initialize_xfm = 'im_affine.aff'
    >>> node.cmdline
    'dti_affine_reg im1.nii im2.nii EDS 4 4 4 0.01 im_affine.aff'
    >>> node.run() # doctest: +SKIP
    """
    _cmd = 'dti_affine_reg'


class DiffeoInputSpec(CommandLineInputSpec):
    fixed_file = File(desc="fixed tensor volume",
                      exists=True, position=0, argstr="%s")
    moving_file = File(desc="moving tensor volume",
                       exists=True, position=1, argstr="%s", copyfile=False)
    mask_file = File(desc="mask", exists=True,  position=2, argstr="%s")
    legacy = traits.Enum(1, desc="legacy parameter; always set to 1",
                         usedefault=True, mandatory=True,
                         position=3, argstr="%d")
    n_iters = traits.Int(6, desc="number of iterations",
                         mandatory=True,
                         position=4, argstr="%d", usedefault=True)
    ftol = traits.Float(0.002, desc="iteration for the optimization to stop",
                        mandatory=True, position=5, argstr="%g",
                        usedefault=True)


class DiffeoOutputSpec(TraitedSpec):
    out_file = File(exists=True)
    out_file_xfm = File(exists=True)


class Diffeo(CommandLineDtitk):
    """Performs diffeomorphic registration between two tensor volumes

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.Diffeo()
    >>> node.inputs.fixed_file = 'im1.nii'
    >>> node.inputs.moving_file = 'im2.nii'
    >>> node.inputs.mask_file = 'mask.nii'
    >>> node.inputs.legacy = 1
    >>> node.inputs.n_iters = 6
    >>> node.inputs.ftol = 0.002
    >>> node.cmdline
    'dti_diffeomorphic_reg im1.nii im2.nii mask.nii 1 6 0.002'
    >>> node.run() # doctest: +SKIP
    """
    input_spec = DiffeoInputSpec
    output_spec = DiffeoOutputSpec
    _cmd = 'dti_diffeomorphic_reg'

    def _list_outputs(self):
        outputs = self.output_spec().get()
        moving = self.inputs.moving_file
        outputs['out_file_xfm'] = fname_presuffix(moving, suffix='_diffeo.df')
        outputs['out_file'] = fname_presuffix(moving, suffix='_diffeo')
        return outputs


class ComposeXfmInputSpec(CommandLineInputSpec):
    in_df = File(desc='diffeomorphic warp file', exists=True,
                 argstr="-df %s", mandatory=True)
    in_aff = File(desc='affine transform file', exists=True,
                  argstr="-aff %s", mandatory=True)
    out_file = File(desc='output path',
                    argstr="-out %s",  genfile=True)


class ComposeXfmOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class ComposeXfm(CommandLineDtitk):
    """
     Combines diffeomorphic and affine transforms

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.ComposeXfm()
    >>> node.inputs.in_df = 'im_warp.df.nii'
    >>> node.inputs.in_aff= 'im_affine.aff'
    >>> node.cmdline
    'dfRightComposeAffine -aff im_affine.aff -df im_warp.df.nii -out
     im_warp_affdf.df.nii'
    >>> node.run() # doctest: +SKIP
    """
    input_spec = ComposeXfmInputSpec
    output_spec = ComposeXfmOutputSpec
    _cmd = 'dfRightComposeAffine'

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_file = self.inputs.out_file
        if not isdefined(out_file):
            out_file = self._gen_filename('out_file')
        outputs['out_file'] = os.path.abspath(out_file)
        return outputs

    def _gen_filename(self, name):
        if name != 'out_file':
            return
        path, base, ext = split_filename(self.inputs.in_df)
        suffix = '_affdf'
        if base.endswith('.df'):
            suffix += '.df'
            base = base[:-3]
        return fname_presuffix(base, suffix=suffix + ext, use_ext=False)


class AffSymTensor3DVolInputSpec(CommandLineInputSpec):
    in_file = File(desc='moving tensor volume', exists=True,
                   argstr="-in %s", mandatory=True)
    out_file = File(desc='output filename',
                    argstr="-out %s", name_source="in_file",
                    name_template="%s_affxfmd", keep_extension=True)
    transform = File(exists=True, argstr="-trans %s",
                     xor=['target', 'translation', 'euler', 'deformation'],
                     desc='transform to apply: specify an input transformation'
                     ' file; parameters input will be ignored',)
    interpolation = traits.Enum('LEI', 'EI', usedefault=True,
                                argstr="-interp %s",
                                desc='Log Euclidean/Euclidean Interpolation')
    reorient = traits.Enum('PPD', 'NO', 'FS', argstr='-reorient %s',
                           usedefault=True, desc='Reorientation strategy: '
                           'preservation of principal direction, no '
                           'reorientation, or finite strain')
    target = File(exists=True, argstr="-target %s", xor=['transform'],
                  desc='output volume specification read from the target '
                  'volume if specified')
    translation = traits.Tuple((traits.Float(), traits.Float(),
                                traits.Float()),
                               desc='translation (x,y,z) in mm',
                               argstr='-translation %g %g %g',
                               xor=['transform'])
    euler = traits.Tuple((traits.Float(), traits.Float(), traits.Float()),
                         desc='(theta, phi, psi) in degrees',
                         xor=['transform'], argstr='-euler %g %g %g')
    deformation = traits.Tuple((traits.Float(),) * 6,
                               desc='(xx,yy,zz,xy,yz,xz)', xor=['transform'],
                               argstr='-deformation %g %g %g %g %g %g')


class AffSymTensor3DVolOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class AffSymTensor3DVol(CommandLineDtitk):
    """
    Applies affine transform to a tensor volume

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.AffSymTensor3DVol()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.transform = 'im_affine.aff'
    >>> node.cmdline
    'affineSymTensor3DVolume -in im1.nii -interp LEI -out im1_affxfmd.nii
     -reorient PPD -trans im_affine.aff'
    >>> node.run() # doctest: +SKIP
    """
    input_spec = AffSymTensor3DVolInputSpec
    output_spec = AffSymTensor3DVolOutputSpec
    _cmd = 'affineSymTensor3DVolume'


class AffScalarVolInputSpec(CommandLineInputSpec):
    in_file = File(desc='moving scalar volume', exists=True,
                   argstr="-in %s", mandatory=True)
    out_file = File(desc='output filename',
                    argstr="-out %s", name_source="in_file",
                    name_template="%s_affxfmd", keep_extension=True)
    transform = File(exists=True, argstr="-trans %s",
                     xor=['target', 'translation', 'euler', 'deformation'],
                     desc='transform to apply: specify an input transformation'
                     ' file; parameters input will be ignored',)
    interpolation = traits.Enum('trilinear', 'NN',
                                usedefault=True, argstr="-interp %s",
                                desc='trilinear or nearest neighbor'
                                ' interpolation')
    target = File(exists=True, argstr="-target %s", xor=['transform'],
                  desc='output volume specification read from the target '
                  'volume if specified')
    translation = traits.Tuple((traits.Float(), traits.Float(),
                                traits.Float()),
                               desc='translation (x,y,z) in mm',
                               argstr='-translation %g %g %g',
                               xor=['transform'])
    euler = traits.Tuple((traits.Float(), traits.Float(), traits.Float()),
                         desc='(theta, phi, psi) in degrees',
                         xor=['transform'], argstr='-euler %g %g %g')
    deformation = traits.Tuple((traits.Float(),) * 6,
                               desc='(xx,yy,zz,xy,yz,xz)', xor=['transform'],
                               argstr='-deformation %g %g %g %g %g %g')


class AffScalarVolOutputSpec(TraitedSpec):
    out_file = File(desc='moved volume', exists=True)


class AffScalarVol(CommandLineDtitk):
    """
    Applies affine transform to a scalar volume

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.AffScalarVol()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.transform = 'im_affine.aff'
    >>> node.cmdline
    'affineScalarVolume -in im1.nii -interp 0 -out im1_affxfmd.nii -trans
     im_affine.aff'
    >>> node.run() # doctest: +SKIP
    """
    input_spec = AffScalarVolInputSpec
    output_spec = AffScalarVolOutputSpec
    _cmd = 'affineScalarVolume'

    def _format_arg(self, name, spec, value):
        if name == 'interpolation':
            value = {'trilinear': 0, 'NN': 1}[value]
        return super(AffScalarVol, self)._format_arg(name, spec, value)


class DiffeoSymTensor3DVolInputSpec(CommandLineInputSpec):
    in_file = File(desc='moving tensor volume', exists=True,
                   argstr="-in %s", mandatory=True)
    out_file = File(desc='output filename',
                    argstr="-out %s", name_source="in_file",
                    name_template="%s_diffeoxfmd", keep_extension=True)
    transform = File(exists=True, argstr="-trans %s",
                     mandatory=True, desc='transform to apply')
    df = traits.Str('FD', argstr="-df %s", usedefault=True)
    interpolation = traits.Enum('LEI', 'EI', usedefault=True,
                                argstr="-interp %s",
                                desc='Log Euclidean/Euclidean Interpolation')
    reorient = traits.Enum('PPD', 'FS', argstr='-reorient %s',
                           usedefault=True, desc='Reorientation strategy: '
                           'preservation of principal direction or finite '
                           'strain')
    target = File(exists=True, argstr="-target %s", xor=['voxel_size'],
                  desc='output volume specification read from the target '
                  'volume if specified')
    voxel_size = traits.Tuple((traits.Float(), traits.Float(), traits.Float()),
                              desc='xyz voxel size (superseded by target)',
                              argstr="-vsize %g %g %g", xor=['target'])
    flip = traits.Tuple((traits.Int(), traits.Int(), traits.Int()),
                        argstr="-flip %d %d %d")
    resampling_type = traits.Enum('backward', 'forward',
                                  desc='use backward or forward resampling',
                                  argstr="-type %s")


class DiffeoSymTensor3DVolOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class DiffeoSymTensor3DVol(CommandLineDtitk):
    """
    Applies diffeomorphic transform to a tensor volume

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.DiffeoSymTensor3DVol()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.transform = 'im_warp.df.nii'
    >>> node.cmdline
    'deformationSymTensor3DVolume -df FD -in im1.nii -interp LEI -out
     im1_diffeoxfmd.nii -reorient PPD -trans im_warp.df.nii'
    >>> node.run() # doctest: +SKIP
    """

    input_spec = DiffeoSymTensor3DVolInputSpec
    output_spec = DiffeoSymTensor3DVolOutputSpec
    _cmd = 'deformationSymTensor3DVolume'

    def _format_arg(self, name, spec, value):
        if name == 'resampling_type':
            value = {'forward': 0, 'backward': 1}[value]
        return super(DiffeoSymTensor3DVol, self)._format_arg(name, spec, value)


class DiffeoScalarVolInputSpec(CommandLineInputSpec):
    in_file = File(desc='moving scalar volume', exists=True,
                   argstr="-in %s", mandatory=True)
    out_file = File(desc='output filename',
                    argstr="-out %s", name_source="in_file",
                    name_template="%s_diffeoxfmd", keep_extension=True)
    transform = File(exists=True, argstr="-trans %s",
                     mandatory=True, desc='transform to apply')
    target = File(exists=True, argstr="-target %s", xor=['voxel_size'],
                  desc='output volume specification read from the target '
                  'volume if specified')
    voxel_size = traits.Tuple((traits.Float(), traits.Float(), traits.Float()),
                              desc='xyz voxel size (superseded by target)',
                              argstr="-vsize %g %g %g", xor=['target'])
    flip = traits.Tuple((traits.Int(), traits.Int(), traits.Int()),
                        argstr="-flip %d %d %d")
    resampling_type = traits.Enum('backward', 'forward',
                                  desc='use backward or forward resampling',
                                  argstr="-type %s")
    interpolation = traits.Enum('trilinear', 'NN',
                                desc='trilinear, or nearest neighbor',
                                argstr="-interp %s",
                                usedefault=True)


class DiffeoScalarVolOutputSpec(TraitedSpec):
    out_file = File(desc='moved volume', exists=True)


class DiffeoScalarVol(CommandLineDtitk):
    """
    Applies diffeomorphic transform to a scalar volume

    Example
    -------

    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.DiffeoScalarVol()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.transform = 'im_warp.df.nii'
    >>> node.cmdline
    'deformationScalarVolume -in im1.nii -interp 0 -out im1_diffeoxfmd.nii
     -trans im_warp.df.nii'
    >>> node.run() # doctest: +SKIP
    """

    input_spec = DiffeoScalarVolInputSpec
    output_spec = DiffeoScalarVolOutputSpec
    _cmd = 'deformationScalarVolume'

    def _format_arg(self, name, spec, value):
        if name == 'resampling_type':
            value = {'forward': 0, 'backward': 1}[value]
        elif name == 'interpolation':
            value = {'trilinear': 0, 'NN': 1}[value]
        return super(DiffeoScalarVol, self)._format_arg(name, spec, value)


class RigidTask(DTITKRenameMixin, Rigid):
    pass


class AffineTask(DTITKRenameMixin, Affine):
    pass


class DiffeoTask(DTITKRenameMixin, Diffeo):
    pass


class ComposeXfmTask(DTITKRenameMixin, ComposeXfm):
    pass


class affScalarVolTask(DTITKRenameMixin, AffScalarVol):
    pass


class affSymTensor3DVolTask(DTITKRenameMixin, AffSymTensor3DVol):
    pass


class diffeoScalarVolTask(DTITKRenameMixin, DiffeoScalarVol):
    pass


class diffeoSymTensor3DVolTask(DTITKRenameMixin, DiffeoSymTensor3DVol):
    pass



"""Utilities to manipulate and search through .pklz crash files."""

import re
import sys
import os.path as op
from glob import glob

from traits.trait_errors import TraitError
from nipype.utils.filemanip import loadcrash


def load_pklz_traceback(crash_filepath):
    """Return the traceback message in the given crash file."""
    try:
        data = loadcrash(crash_filepath)
    except TraitError as te:
        return str(te)
    except:
        raise
    else:
        return '\n'.join(data['traceback'])


def iter_tracebacks(logdir):
    """Return an iterator over each file path and
    traceback field inside `logdir`.
    Parameters
    ----------
    logdir: str
        Path to the log folder.

    field: str
        Field name to be read from the crash file.

    Yields
    ------
    path_file: str

    traceback: str
    """
    crash_files = sorted(glob(op.join(logdir, '*.pkl*')))

    for cf in crash_files:
        yield cf, load_pklz_traceback(cf)


def display_crash_file(crashfile, rerun, debug, directory):
    """display crash file content and rerun if required"""
    from nipype.utils.filemanip import loadcrash

    crash_data = loadcrash(crashfile)
    node = None
    if 'node' in crash_data:
        node = crash_data['node']
    tb = crash_data['traceback']
    print("\n")
    print("File: %s" % crashfile)

    if node:
        print("Node: %s" % node)
        if node.base_dir:
            print("Working directory: %s" % node.output_dir())
        else:
            print("Node crashed before execution")
        print("\n")
        print("Node inputs:")
        print(node.inputs)
        print("\n")
    print("Traceback: ")
    print(''.join(tb))
    print("\n")

    if rerun:
        if node is None:
            print("No node in crashfile. Cannot rerun")
            return
        print("Rerunning node")
        node.base_dir = directory
        node.config = {'execution': {'crashdump_dir': '/tmp'}}
        try:
            node.run()
        except:
            if debug and debug != 'ipython':
                import pdb
                pdb.post_mortem()
            else:
                raise
        print("\n")



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import FAST


def test_FAST_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bias_iters=dict(argstr='-I %d', ),
        bias_lowpass=dict(
            argstr='-l %d',
            units='mm',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hyper=dict(argstr='-H %.2f', ),
        img_type=dict(argstr='-t %d', ),
        in_files=dict(
            argstr='%s',
            copyfile=False,
            mandatory=True,
            position=-1,
        ),
        init_seg_smooth=dict(argstr='-f %.3f', ),
        init_transform=dict(
            argstr='-a %s',
            extensions=None,
        ),
        iters_afterbias=dict(argstr='-O %d', ),
        manual_seg=dict(
            argstr='-s %s',
            extensions=None,
        ),
        mixel_smooth=dict(argstr='-R %.2f', ),
        no_bias=dict(argstr='-N', ),
        no_pve=dict(argstr='--nopve', ),
        number_classes=dict(argstr='-n %d', ),
        other_priors=dict(argstr='-A %s', ),
        out_basename=dict(
            argstr='-o %s',
            extensions=None,
        ),
        output_biascorrected=dict(argstr='-B', ),
        output_biasfield=dict(argstr='-b', ),
        output_type=dict(),
        probability_maps=dict(argstr='-p', ),
        segment_iters=dict(argstr='-W %d', ),
        segments=dict(argstr='-g', ),
        use_priors=dict(argstr='-P', ),
        verbose=dict(argstr='-v', ),
    )
    inputs = FAST.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FAST_outputs():
    output_map = dict(
        bias_field=dict(),
        mixeltype=dict(extensions=None, ),
        partial_volume_files=dict(),
        partial_volume_map=dict(extensions=None, ),
        probability_maps=dict(),
        restored_image=dict(),
        tissue_class_files=dict(),
        tissue_class_map=dict(extensions=None, ),
    )
    outputs = FAST.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
